@@ -1,9 +1,14 @@
+/**
+ * @file
+ * @brief API colorprocs.h
+ * @ingroup common_utils
+ */
 /*************************************************************************
  * Copyright (c) 2011 AT&T Intellectual Property 
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
@@ -14,18 +19,16 @@
 #include <stdbool.h>
 #include <stdlib.h>
 #include <string.h>
-#include <ctype.h>
-
 #include <common/arith.h>
 #include <common/color.h>
 #include <common/colorprocs.h>
 #include <common/colortbl.h>
-#include <common/memory.h>
 #include <cgraph/agxbuf.h>
-#include <cgraph/alloc.h>
 #include <cgraph/gv_ctype.h>
-#include <cgraph/strcasecmp.h>
-#include <cgraph/unreachable.h>
+#include <cgraph/gv_math.h>
+#include <util/alloc.h>
+#include <util/strcasecmp.h>
+#include <util/unreachable.h>
 
 static char* colorscheme;
 
@@ -44,7 +47,7 @@ static void hsv2rgb(double h, double s, double v,
 	    h = 0.0;
 	h = 6.0 * h;
 	i = (int) h;
-	f = h - (double) i;
+	f = h - i;
 	p = v * (1 - s);
 	q = v * (1 - s * f);
 	t = v * (1 - s * (1 - f));
@@ -103,11 +106,11 @@ static void rgb2hsv(double r, double g, double b,
 	rc = (rgbmax - r) / (rgbmax - rgbmin);
 	gc = (rgbmax - g) / (rgbmax - rgbmin);
 	bc = (rgbmax - b) / (rgbmax - rgbmin);
-	if (r == rgbmax)
+	if (is_exactly_equal(r, rgbmax))
 	    ht = bc - gc;
-	else if (g == rgbmax)
+	else if (is_exactly_equal(g, rgbmax))
 	    ht = 2 + rc - bc;
-	else if (b == rgbmax)
+	else if (is_exactly_equal(b, rgbmax))
 	    ht = 4 + gc - rc;
 	ht = ht * 60.0;
 	if (ht < 0.0)
@@ -118,46 +121,9 @@ static void rgb2hsv(double r, double g, double b,
     *s = st;
 }
 
-static void rgb2cmyk(double r, double g, double b, double *c, double *m,
-		     double *y, double *k)
-{
-    *c = 1.0 - r;
-    *m = 1.0 - g;
-    *y = 1.0 - b;
-    *k = fmin(*c, *m);
-    *k = fmin(*y, *k);
-    *c -= *k;
-    *m -= *k;
-    *y -= *k;
-}
-
 static int colorcmpf(const void *p0, const void *p1)
 {
-    return strcasecmp(((const hsvrgbacolor_t *) p0)->name, ((const hsvrgbacolor_t *) p1)->name);
-}
-
-char *canontoken(char *str)
-{
-    static char *canon;
-    static size_t allocated;
-    char c, *p, *q;
-    size_t len;
-
-    p = str;
-    len = strlen(str);
-    if (len >= allocated) {
-	size_t new_allocated = len + 1 + 10;
-	canon = gv_recalloc(canon, allocated, new_allocated, sizeof(char));
-	allocated = new_allocated;
-    }
-    q = canon;
-    while ((c = *p++)) {
-	if (gv_isupper(c))
-	    c = (char)tolower(c);
-	*q++ = c;
-    }
-    *q = '\0';
-    return canon;
+  return strcasecmp(p0, ((const hsvrgbacolor_t *)p1)->name);
 }
 
 /* fullColor:
@@ -204,19 +170,17 @@ static char *fullColor(agxbuf *xb, const char *prefix, const char *str) {
 #define DFLT_SCHEME_LEN ((sizeof(DFLT_SCHEME)-1)/sizeof(char))
 #define ISNONDFLT(s) ((s) && *(s) && strncasecmp(DFLT_SCHEME, s, DFLT_SCHEME_LEN-1))
 
-static char* resolveColor (char* str)
-{
-    char* s;
-    char* ss;   /* second slash */
-    char* c2;   /* second char */
+static char *resolveColor(const char *str) {
+    const char *s;
 
-    if (!strcmp(str, "black")) return str;
-    if (!strcmp(str, "white")) return str;
-    if (!strcmp(str, "lightgrey")) return str;
+    if (!strcmp(str, "black")) return strdup(str);
+    if (!strcmp(str, "white")) return strdup(str);
+    if (!strcmp(str, "lightgrey")) return strdup(str);
     agxbuf xb = {0};
     if (*str == '/') {   /* if begins with '/' */
-	c2 = str+1;
-        if ((ss = strchr(c2, '/'))) {  /* if has second '/' */
+	const char *const c2 = str + 1; // second char
+        const char *const ss = strchr(c2, '/'); // second slash
+        if (ss != NULL) { // if has second '/'
 	    if (*c2 == '/') {    /* if second '/' is second character */
 		    /* Do not compare against final '/' */
 		if (ISNONDFLT(colorscheme))
@@ -231,19 +195,15 @@ static char* resolveColor (char* str)
     }
     else if (ISNONDFLT(colorscheme)) s = fullColor(&xb, colorscheme, str);
     else s = str;
-    s = canontoken(s);
+    char *on_heap = strdup(s);
     agxbfree(&xb);
-    return s;
+    return on_heap;
 }
 
-int colorxlate(char *str, gvcolor_t * color, color_type_t target_type)
-{
+int colorxlate(const char *str, gvcolor_t *color, color_type_t target_type) {
     static hsvrgbacolor_t *last;
-    char *p;
-    hsvrgbacolor_t fake;
     char c;
     double H, S, V, A, R, G, B;
-    double C, M, Y, K;
     unsigned int r, g, b, a;
     int rc;
 
@@ -251,7 +211,7 @@ int colorxlate(char *str, gvcolor_t * color, color_type_t target_type)
 
     rc = COLOR_OK;
     for (; *str == ' '; str++);	/* skip over any leading whitespace */
-    p = str;
+    const char *p = str;
 
     /* test for rgb value such as: "#ff0000"
        or rgba value such as "#ff000080" */
@@ -284,16 +244,6 @@ int colorxlate(char *str, gvcolor_t * color, color_type_t target_type)
 	    color->u.rgba[2] = (unsigned char)b;
 	    color->u.rgba[3] = (unsigned char)a;
 	    break;
-	case CMYK_BYTE:
-	    R = (double) r / 255.0;
-	    G = (double) g / 255.0;
-	    B = (double) b / 255.0;
-	    rgb2cmyk(R, G, B, &C, &M, &Y, &K);
-	    color->u.cmyk[0] = (int) C *255;
-	    color->u.cmyk[1] = (int) M *255;
-	    color->u.cmyk[2] = (int) Y *255;
-	    color->u.cmyk[3] = (int) K *255;
-	    break;
 	case RGBA_WORD:
 	    color->u.rrggbbaa[0] = (int)(r * 65535 / 255);
 	    color->u.rrggbbaa[1] = (int)(g * 65535 / 255);
@@ -339,18 +289,10 @@ int colorxlate(char *str, gvcolor_t * color, color_type_t target_type)
 		break;
 	    case RGBA_BYTE:
 		hsv2rgb(H, S, V, &R, &G, &B);
-		color->u.rgba[0] = (int) (R * 255);
-		color->u.rgba[1] = (int) (G * 255);
-		color->u.rgba[2] = (int) (B * 255);
-		color->u.rgba[3] = (int) (A * 255);
-		break;
-	    case CMYK_BYTE:
-		hsv2rgb(H, S, V, &R, &G, &B);
-		rgb2cmyk(R, G, B, &C, &M, &Y, &K);
-		color->u.cmyk[0] = (int) C *255;
-		color->u.cmyk[1] = (int) M *255;
-		color->u.cmyk[2] = (int) Y *255;
-		color->u.cmyk[3] = (int) K *255;
+		color->u.rgba[0] = (unsigned char)(R * 255);
+		color->u.rgba[1] = (unsigned char)(G * 255);
+		color->u.rgba[2] = (unsigned char)(B * 255);
+		color->u.rgba[3] = (unsigned char)(A * 255);
 		break;
 	    case RGBA_WORD:
 		hsv2rgb(H, S, V, &R, &G, &B);
@@ -380,13 +322,14 @@ int colorxlate(char *str, gvcolor_t * color, color_type_t target_type)
     }
 
     /* test for known color name (generic, not renderer specific known names) */
-    fake.name = resolveColor(str);
-    if (!fake.name)
+    char *name = resolveColor(str);
+    if (!name)
 	return COLOR_MALLOC_FAIL;
-    if (last == NULL || strcmp(last->name, fake.name)) {
-	last = bsearch(&fake, color_lib, sizeof(color_lib) / sizeof(hsvrgbacolor_t),
-	               sizeof(fake), colorcmpf);
+    if (last == NULL || strcasecmp(last->name, name)) {
+	last = bsearch(name, color_lib, sizeof(color_lib) / sizeof(hsvrgbacolor_t),
+	               sizeof(color_lib[0]), colorcmpf);
     }
+    free(name);
     if (last != NULL) {
 	switch (target_type) {
 	case HSVA_DOUBLE:
@@ -401,16 +344,6 @@ int colorxlate(char *str, gvcolor_t * color, color_type_t target_type)
 	    color->u.rgba[2] = last->b;
 	    color->u.rgba[3] = last->a;
 	    break;
-	case CMYK_BYTE:
-	    R = last->r / 255.0;
-	    G = last->g / 255.0;
-	    B = last->b / 255.0;
-	    rgb2cmyk(R, G, B, &C, &M, &Y, &K);
-	    color->u.cmyk[0] = (int) C * 255;
-	    color->u.cmyk[1] = (int) M * 255;
-	    color->u.cmyk[2] = (int) Y * 255;
-	    color->u.cmyk[3] = (int) K * 255;
-	    break;
 	case RGBA_WORD:
 	    color->u.rrggbbaa[0] = last->r * 65535 / 255;
 	    color->u.rrggbbaa[1] = last->g * 65535 / 255;
@@ -444,10 +377,6 @@ int colorxlate(char *str, gvcolor_t * color, color_type_t target_type)
 	color->u.rgba[0] = color->u.rgba[1] = color->u.rgba[2] = 0;
 	color->u.rgba[3] = 255;	/* opaque */
 	break;
-    case CMYK_BYTE:
-	color->u.cmyk[0] =
-	    color->u.cmyk[1] = color->u.cmyk[2] = color->u.cmyk[3] = 0;
-	break;
     case RGBA_WORD:
 	color->u.rrggbbaa[0] = color->u.rrggbbaa[1] = color->u.rrggbbaa[2] = 0;
 	color->u.rrggbbaa[3] = 65535;	/* opaque */
@@ -466,10 +395,8 @@ int colorxlate(char *str, gvcolor_t * color, color_type_t target_type)
     return rc;
 }
 
-/* setColorScheme:
- * Set current color scheme for resolving names.
- */
-void setColorScheme (char* s)
-{
-    colorscheme = s;
+char *setColorScheme(const char *s) {
+  char *previous = colorscheme;
+  colorscheme = s == NULL ? NULL : gv_strdup(s);
+  return previous;
 }
