@@ -3,7 +3,7 @@
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
@@ -17,11 +17,14 @@
 
 #include "config.h"
 #include <cgraph/agxbuf.h>
-#include <ctype.h>
+#include <cgraph/gv_ctype.h>
 #include <expr/exlib.h>
+#include <stdbool.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <util/streq.h>
+#include <util/unreachable.h>
 
 #if defined(TRACE_lex) && TRACE_lex
 
@@ -34,6 +37,7 @@ trace(Expr_t* ex, int lev, char* op, int c)
 {
 	char*	s = 0;
 	char*	t;
+	bool free_t = false;
 	char	buf[16];
 	void*	x = 0;
 
@@ -94,7 +98,7 @@ trace(Expr_t* ex, int lev, char* op, int c)
 		break;
 	case INTEGER:
 		s = " INTEGER ";
-		snprintf(t = buf, sizeof(buf), "%lld", (long long)ex_lval.integer);
+		snprintf(t = buf, sizeof(buf), "%lld", ex_lval.integer);
 		break;
 	case LABEL:
 		s = " LABEL ";
@@ -128,6 +132,7 @@ trace(Expr_t* ex, int lev, char* op, int c)
 	case STRING:
 		s = " STRING ";
 		t = fmtesc(ex_lval.string);
+		free_t = true;
 		break;
 	case UNSIGNED:
 		s = " UNSIGNED ";
@@ -154,7 +159,7 @@ trace(Expr_t* ex, int lev, char* op, int c)
 	case FOR:
 		s = " for";
 		break;
-	case ITERATER:
+	case ITERATOR:
 		s = " forf";
 		break;
 	case GSUB:
@@ -218,6 +223,7 @@ trace(Expr_t* ex, int lev, char* op, int c)
 			*s++ = c;
 			*s = 0;
 			t = fmtesc(buf);
+			free_t = true;
 			s = " ";
 		}
 		break;
@@ -227,6 +233,9 @@ trace(Expr_t* ex, int lev, char* op, int c)
 
 	else
 		error(TRACE_lex + lev, "%s: [%d] %04d%s%s", op, ex->input->nesting, c, s, t);
+	if (free_t) {
+		free(t);
+	}
 }
 
 /*
@@ -274,16 +283,6 @@ lex(Expr_t* ex)
 				continue;
 			}
 		}
-		else if (ex->input->sp)
-		{
-			if (!(c = *ex->input->sp++))
-			{
-				if (!expop(ex))
-					continue;
-				else trace(ex, -1, "expop sp FAIL", 0);
-				ex->input->sp--;
-			}
-		}
 		else if (ex->input->fp)
 		{
 			if ((c = getc(ex->input->fp)) == EOF)
@@ -293,13 +292,6 @@ lex(Expr_t* ex)
 				else trace(ex, -1, "expop fp FAIL", 0);
 				c = 0;
 			}
-			else if ((ex->disc->flags & EX_INTERACTIVE) && c == '\n' && ex->input->next && !ex->input->next->next && ex->input->nesting <= 0)
-			{
-				error_info.line++;
-				expop(ex);
-				trace(ex, -1, "expop sp FORCE", 0);
-				c = 0;
-			}
 		}
 		else c = 0;
 		if (c == '\n')
@@ -449,14 +441,15 @@ extoken_fn(Expr_t* ex)
 			case '!':
 				c = NE;
 				break;
+			default:
+				UNREACHABLE();
 			}
 			else exunlex(ex, q);
 			return ex_lval.op = c;
 		case '#':
-			if (!ex->linewrap && !(ex->disc->flags & EX_PURE))
-			{
+			if (!ex->linewrap) {
 				s = ex->linep - 1;
-				while (s > ex->line && isspace((int)*(s - 1)))
+				while (s > ex->line && gv_isspace(*(s - 1)))
 					s--;
 				if (s == ex->line)
 				{
@@ -522,7 +515,7 @@ extoken_fn(Expr_t* ex)
 			ex_lval.integer = chrtoi(s);
 			return INTEGER;
 		case '.':
-			if (isdigit(c = lex(ex)))
+			if (gv_isdigit(c = lex(ex)))
 			{
 				agxbclear(&ex->tmp);
 				agxbput(&ex->tmp, "0.");
@@ -540,14 +533,14 @@ extoken_fn(Expr_t* ex)
 			{
 				b = 16;
 				agxbputc(&ex->tmp, (char)c);
-				for (c = lex(ex); isxdigit(c); c = lex(ex))
+				for (c = lex(ex); gv_isxdigit(c); c = lex(ex))
 				{
 					agxbputc(&ex->tmp, (char)c);
 				}
 			}
 			else
 			{
-				while (isdigit(c))
+				while (gv_isdigit(c))
 				{
 					agxbputc(&ex->tmp, (char)c);
 					c = lex(ex);
@@ -558,7 +551,7 @@ extoken_fn(Expr_t* ex)
 					do
 					{
 						agxbputc(&ex->tmp, (char)c);
-					} while (isalnum(c = lex(ex)));
+					} while (gv_isalnum(c = lex(ex)));
 				}
 				else
 				{
@@ -567,7 +560,7 @@ extoken_fn(Expr_t* ex)
 					floating:
 						q = FLOATING;
 						agxbputc(&ex->tmp, (char)c);
-						while (isdigit(c = lex(ex)))
+						while (gv_isdigit(c = lex(ex)))
 							agxbputc(&ex->tmp, (char)c);
 					}
 					if (c == 'e' || c == 'E')
@@ -579,7 +572,7 @@ extoken_fn(Expr_t* ex)
 							agxbputc(&ex->tmp, (char)c);
 							c = lex(ex);
 						}
-						while (isdigit(c))
+						while (gv_isdigit(c))
 						{
 							agxbputc(&ex->tmp, (char)c);
 							c = lex(ex);
@@ -602,7 +595,7 @@ extoken_fn(Expr_t* ex)
 					ex_lval.integer = strtoll(s, &e, b);
 			}
 			exunlex(ex, c);
-			if (*e || isalpha(c) || c == '_' || c == '$')
+			if (*e || gv_isalpha(c) || c == '_' || c == '$')
 			{
 				exerror("%s: invalid numeric constant", s);
 				goto eof;
@@ -610,11 +603,11 @@ extoken_fn(Expr_t* ex)
 			return q;
 		}
 		default:
-			if (isalpha(c) || c == '_' || c == '$')
+			if (gv_isalpha(c) || c == '_' || c == '$')
 			{
 				agxbclear(&ex->tmp);
 				agxbputc(&ex->tmp, (char)c);
-				while (isalnum(c = lex(ex)) || c == '_' || c == '$')
+				while (gv_isalnum(c = lex(ex)) || c == '_' || c == '$')
 					agxbputc(&ex->tmp, (char)c);
 				exunlex(ex, c);
 				s = agxbuse(&ex->tmp);
@@ -625,14 +618,14 @@ extoken_fn(Expr_t* ex)
 					dtview(ex->symbols, v);
 				if (!ex_lval.id)
 				{
-					if (!(ex_lval.id = calloc(1, sizeof(Exid_t) + strlen(s) - EX_NAMELEN + 1)))
-					{
+					const size_t size = sizeof(Exid_t) + strlen(s) - EX_NAMELEN + 1;
+					if (!(ex_lval.id = vmalloc(ex->vm, size))) {
 						exnospace();
 						goto eof;
 					}
+					memset(ex_lval.id, 0, size);
 					strcpy(ex_lval.id->name, s);
 					ex_lval.id->lex = NAME;
-					expr.statics += ex_lval.id->isstatic = expr.instatic;
 
 					/*
 					 * LABELs are in the parent scope!
@@ -731,6 +724,8 @@ extoken_fn(Expr_t* ex)
 								continue;
 							}
 							break;
+						default: // ignore; keep consuming characters
+							break;
 						}
 						if (!b++)
 							goto eof;
@@ -785,6 +780,8 @@ extoken_fn(Expr_t* ex)
 					case '[':
 						pc = ']';
 						break;
+					default:
+						UNREACHABLE();
 					}
 					n++;
 				}
