@@ -3,7 +3,7 @@
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
@@ -16,15 +16,18 @@
 
 #include <assert.h>
 #include <cgraph/agxbuf.h>
-#include <cgraph/alloc.h>
-#include <cgraph/exit.h>
 #include <cgraph/gv_ctype.h>
+#include <cgraph/list.h>
 #include <cgraph/startswith.h>
 #include <gvc/gvconfig.h>
 #include <stdbool.h>
+#include <stdio.h>
 #include <stdlib.h>
 #include	<string.h>
 #include <unistd.h>
+#include <util/alloc.h>
+#include <util/exit.h>
+#include <util/gv_fopen.h>
 
 #ifdef ENABLE_LTDL
 #ifdef HAVE_DL_ITERATE_PHDR
@@ -53,7 +56,6 @@ static int glob (GVC_t * gvc, char*, int, int (*errfunc)(const char *, int), glo
 #include <mach-o/dyld.h>
 #endif
 
-#include        <common/memory.h>
 #include        <common/const.h>
 #include        <common/types.h>
 
@@ -97,7 +99,7 @@ extern void textfont_dict_open(GVC_t *gvc);
 static gvplugin_package_t * gvplugin_package_record(GVC_t * gvc,
                                                     const char *package_path,
                                                     const char *name) {
-    gvplugin_package_t *package = gmalloc(sizeof(gvplugin_package_t));
+    gvplugin_package_t *package = gv_alloc(sizeof(gvplugin_package_t));
     package->path = package_path ? gv_strdup(package_path) : NULL;
     package->name = gv_strdup(name);
     package->next = gvc->packages;
@@ -191,7 +193,7 @@ static int gvconfig_plugin_install_from_config(GVC_t * gvc, char *s)
 	    const char *api = token(&nest, &s);
 	    const api_t gv_api = gvplugin_api(api);
 	    if (gv_api == (api_t)-1) {
-		agerr(AGERR, "config error: %s %s not found\n", package_path, api);
+		agerrorf("config error: %s %s not found\n", package_path, api);
 		return 0;
 	    }
 	    do {
@@ -203,7 +205,7 @@ static int gvconfig_plugin_install_from_config(GVC_t * gvc, char *s)
 		        quality = 0;
 		    bool rc = gvplugin_install(gvc, gv_api, type, quality, package, NULL);
 		    if (!rc) {
-		        agerr(AGERR, "config error: %s %s %s\n", package_path, api, type);
+		        agerrorf("config error: %s %s %s\n", package_path, api, type);
 		        return 0;
 		    }
 		}
@@ -246,16 +248,15 @@ static void gvconfig_plugin_install_builtins(GVC_t * gvc)
 static void gvconfig_write_library_config(GVC_t *gvc, char *lib_path,
                                           gvplugin_library_t *library,
                                           FILE *f) {
-    gvplugin_api_t *apis;
     gvplugin_installed_t *types;
-    int i;
 
     fprintf(f, "%s %s {\n", lib_path, library->packagename);
-    for (apis = library->apis; (types = apis->types); apis++) {
+    for (gvplugin_api_t *apis = library->apis; (types = apis->types); apis++) {
         fprintf(f, "\t%s {\n", gvplugin_api_name(apis->api));
-	for (i = 0; types[i].type; i++) {
+	for (size_t i = 0; types[i].type; i++) {
 	    /* verify that dependencies are available */
-            if (! (gvplugin_load(gvc, apis->api, types[i].type)))
+            if (!gvplugin_load(gvc, apis->api, types[i].type,
+                               gvc->common.verbose > 0 ? f : NULL))
 		fprintf(f, "#FAILS");
 	    fprintf(f, "\t\t%s %d\n", types[i].type, types[i].quality);
 	}
@@ -301,17 +302,17 @@ char * gvconfig_libdir(GVC_t * gvc)
 		
 		MEMORY_BASIC_INFORMATION mbi;
 		if (VirtualQuery (&gvconfig_libdir, &mbi, sizeof(mbi)) == 0) {
-		agerr(AGERR,"failed to get handle for executable.\n");
+		agerrorf("failed to get handle for executable.\n");
 		return 0;
 	    }
 	    r = GetModuleFileName ((HMODULE)mbi.AllocationBase, line, BSZ);
 	    if (!r || (r == BSZ)) {
-		agerr(AGERR,"failed to get path for executable.\n");
+		agerrorf("failed to get path for executable.\n");
 		return 0;
 	    }
 	    s = strrchr(line,'\\');
 	    if (!s) {
-		agerr(AGERR,"no slash in path %s.\n", line);
+		agerrorf("no slash in path %s.\n", line);
 		return 0;
 	    }
 	    *s = '\0';
@@ -339,7 +340,7 @@ char * gvconfig_libdir(GVC_t * gvc)
 		    if (len < BSZ)
 			libdir = line;
 		    else
-		        libdir = gmalloc(len);
+		        libdir = gv_alloc(len);
 		    if (ind > 0) {
 		        memmove(libdir, p, ind);
 		    }
@@ -352,7 +353,7 @@ char * gvconfig_libdir(GVC_t * gvc)
 	    dl_iterate_phdr(line_callback, line);
 	    libdir = line;
 #else
-	    FILE* f = fopen ("/proc/self/maps", "r");
+	    FILE* f = gv_fopen("/proc/self/maps", "r");
 	    if (f) {
 		while (!feof (f)) {
 		    if (!fgets (line, sizeof (line), f))
@@ -490,9 +491,9 @@ static void config_rescan(GVC_t *gvc, char *config_path)
 #endif
 
     if (config_path) {
-	f = fopen(config_path,"w");
+	f = gv_fopen(config_path, "w");
 	if (!f) {
-	    agerr(AGERR,"failed to open %s for write.\n", config_path);
+	    agerrorf("failed to open %s for write.\n", config_path);
 	    graphviz_exit(1);
 	}
 
@@ -596,19 +597,19 @@ void gvconfig(GVC_t * gvc, bool rescan)
     	    goto done;
         }
         else {
-    	    f = fopen(gvc->config_path,"r");
+    	    f = gv_fopen(gvc->config_path, "r");
     	    if (!f) {
-    	        agerr (AGERR,"failed to open %s for read.\n", gvc->config_path);
+    	        agerrorf("failed to open %s for read.\n", gvc->config_path);
 		return;
     	    }
     	    else if (config_st.st_size == 0) {
-    	        agerr(AGERR, "%s is zero sized.\n", gvc->config_path);
+    	        agerrorf("%s is zero sized.\n", gvc->config_path);
     	    }
     	    else {
     	        config_text = gv_alloc((size_t)config_st.st_size + 1);
     	        size_t sz = fread(config_text, 1, (size_t)config_st.st_size, f);
     	        if (sz == 0) {
-    	            agerr(AGERR, "%s read error.\n", gvc->config_path);
+    	            agerrorf("%s read error.\n", gvc->config_path);
     	        }
     	        else {
     	            gvc->config_found = true;
@@ -633,6 +634,8 @@ done:
 
 /* Emulating windows glob */
 
+DEFINE_LIST_WITH_DTOR(strs, char *, free)
+
 /* glob:
  * Assumes only GLOB_NOSORT flag given. That is, there is no offset,
  * and no previous call to glob.
@@ -644,9 +647,7 @@ glob (GVC_t* gvc, char* pattern, int flags, int (*errfunc)(const char *, int), g
     char* libdir;
     WIN32_FIND_DATA wfd;
     HANDLE h;
-    char** str=0;
-    int arrsize=0;
-    int cnt = 0;
+    strs_t strs = {0};
     
     pglob->gl_pathc = 0;
     pglob->gl_pathv = NULL;
@@ -655,30 +656,29 @@ glob (GVC_t* gvc, char* pattern, int flags, int (*errfunc)(const char *, int), g
     if (h == INVALID_HANDLE_VALUE) return GLOB_NOMATCH;
     libdir = gvconfig_libdir(gvc);
     do {
-      if (cnt >= arrsize-1) {
-        arrsize += 512;
-        char **new_str = realloc(str, arrsize * sizeof(char*));
-        if (!new_str) goto oom;
-        str = new_str;
+      const size_t size =
+        strlen(libdir) + strlen(DIRSEP) + strlen(wfd.cFileName) + 1;
+      char *const entry = malloc(size);
+      if (!entry) {
+        goto oom;
+      }
+      snprintf(entry, size, "%s%s%s", libdir, DIRSEP, wfd.cFileName);
+      if (strs_try_append(&strs, entry) != 0) {
+        free(entry);
+        goto oom;
       }
-      str[cnt] = malloc (strlen(libdir)+1+strlen(wfd.cFileName)+1);
-      if (!str[cnt]) goto oom;
-      strcpy(str[cnt],libdir);
-      strcat(str[cnt],DIRSEP);
-      strcat(str[cnt],wfd.cFileName);
-      cnt++;
     } while (FindNextFile (h, &wfd));
-    str[cnt] = 0;
+    if (strs_try_append(&strs, NULL) != 0) {
+      goto oom;
+    }
 
-    pglob->gl_pathc = cnt;
-    pglob->gl_pathv = realloc(str, (cnt+1)*sizeof(char*));
+    pglob->gl_pathc = strs_size(&strs);
+    pglob->gl_pathv = strs_detach(&strs);
     
     return 0;
 
 oom:
-    for (int i = 0; i < cnt; ++i)
-      free(str[i]);
-    free(str);
+    strs_free(&strs);
     return GLOB_NOSPACE;
 }
 
