@@ -3,7 +3,7 @@
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
@@ -14,18 +14,17 @@
  */
 
 #include <assert.h>
-#include <cgraph/alloc.h>
+#include <cgraph/gv_ctype.h>
 #include <neatogen/neato.h>
 #include <cgraph/agxbuf.h>
+#include <cgraph/startswith.h>
 #include <common/utils.h>
-#include <ctype.h>
 #include <float.h>
 #include <math.h>
 #include <neatogen/voronoi.h>
 #include <neatogen/info.h>
 #include <neatogen/edges.h>
 #include <neatogen/site.h>
-#include <neatogen/heap.h>
 #include <neatogen/hedges.h>
 #include <neatogen/digcola.h>
 #if ((defined(HAVE_GTS) || defined(HAVE_TRIANGLE)) && defined(SFDP))
@@ -36,63 +35,55 @@
 #include <vpsc/csolve_VPSC.h>
 #include <neatogen/quad_prog_vpsc.h>
 #endif
-#include <cgraph/strcasecmp.h>
 #include <stddef.h>
+#include <util/alloc.h>
+#include <util/strcasecmp.h>
 
-#define SEPFACT         0.8f  /* default esep/sep */
+#define SEPFACT         0.8 // default esep/sep
 
-static double margin = 0.05;	/* Create initial bounding box by adding
-				 * margin * dimension around box enclosing
-				 * nodes.
-				 */
-static double incr = 0.05;	/* Increase bounding box by adding
+static const double incr = 0.05;	/* Increase bounding box by adding
 				 * incr * dimension around box.
 				 */
-static int iterations = -1;	/* Number of iterations */
-static int useIter = 0;		/* Use specified number of iterations */
 
-static bool doAll = false; // Move all nodes, regardless of overlap
-static Site **sites;		/* Array of pointers to sites; used in qsort */
-static Site **endSite;		/* Sentinel on sites array */
-static Point nw, ne, sw, se;	/* Corners of clipping window */
+typedef struct {
+  Site **sites; ///< array of pointers to sites; used in qsort
+  Site **endSite; ///< sentinel on sites array
+  Point nw, ne, sw, se; ///< corners of clipping window
+  Site **nextSite;
+} state_t;
 
-static Site **nextSite;
-
-static void setBoundBox(Point * ll, Point * ur)
-{
+static void setBoundBox(state_t *st, Point *ll, Point *ur) {
     pxmin = ll->x;
     pxmax = ur->x;
     pymin = ll->y;
     pymax = ur->y;
-    nw.x = sw.x = pxmin;
-    ne.x = se.x = pxmax;
-    nw.y = ne.y = pymax;
-    sw.y = se.y = pymin;
+    st->nw.x = st->sw.x = pxmin;
+    st->ne.x = st->se.x = pxmax;
+    st->nw.y = st->ne.y = pymax;
+    st->sw.y = st->se.y = pymin;
 }
 
- /* freeNodes:
-  * Free node resources.
-  */
+/// Free node resources.
 static void freeNodes(void)
 {
     for (size_t i = 0; i < nsites; i++) {
 	breakPoly(&nodeInfo[i].poly);
     }
     polyFree();
-    infoinit();			/* Free vertices */
+    if (nodeInfo != NULL) {
+	free(nodeInfo->verts); // Free vertices
+    }
     free(nodeInfo);
 }
 
-/* chkBoundBox:
- *   Compute extremes of graph, then set up bounding box.
+/*   Compute extremes of graph, then set up bounding box.
  *   If user supplied a bounding box, use that;
  *   else if "window" is a graph attribute, use that; 
  *   otherwise, define bounding box as a percentage expansion of
  *   graph extremes.
  *   In the first two cases, check that graph fits in bounding box.
  */
-static void chkBoundBox(Agraph_t * graph)
-{
+static void chkBoundBox(state_t *st, Agraph_t *graph) {
     Point ll, ur;
 
     double x_min = DBL_MAX;
@@ -111,10 +102,10 @@ static void chkBoundBox(Agraph_t * graph)
 	y_max = fmax(y_max, pp->corner.y + y);
     }
 
+    // create initial bounding box by adding margin × dimension around box
+    // enclosing nodes
     char *marg = agget(graph, "voro_margin");
-    if (marg && *marg != '\0') {
-	margin = atof(marg);
-    }
+    const double margin = (marg && *marg != '\0') ? atof(marg) : 0.05;
     double ydelta = margin * (y_max - y_min);
     double xdelta = margin * (x_max - x_min);
     ll.x = x_min - xdelta;
@@ -122,17 +113,13 @@ static void chkBoundBox(Agraph_t * graph)
     ur.x = x_max + xdelta;
     ur.y = y_max + ydelta;
 
-    setBoundBox(&ll, &ur);
+    setBoundBox(st, &ll, &ur);
 }
 
- /* makeInfo:
-  * For each node in the graph, create a Info data structure 
-  */
+ /// For each node in the graph, create a Info data structure 
 static int makeInfo(Agraph_t * graph)
 {
-    Agnode_t *node;
-    expand_t pmargin;
-    int (*polyf)(Poly *, Agnode_t *, float, float);
+    int (*polyf)(Poly *, Agnode_t *, double, double);
 
     assert(agnnodes(graph) >= 0);
     nsites = (size_t)agnnodes(graph);
@@ -140,9 +127,9 @@ static int makeInfo(Agraph_t * graph)
 
     nodeInfo = gv_calloc(nsites, sizeof(Info_t));
 
-    node = agfstnode(graph);
+    Agnode_t *node = agfstnode(graph);
 
-    pmargin = sepFactor (graph);
+    expand_t pmargin = sepFactor (graph);
 
     if (pmargin.doAdd) {
 	polyf = makeAddPoly;
@@ -167,6 +154,7 @@ static int makeInfo(Agraph_t * graph)
 	ip->site.refcnt = 1;
 	ip->node = node;
 	ip->verts = NULL;
+	ip->n_verts = 0;
 	node = agnxtnode(graph, node);
     }
     return 0;
@@ -175,10 +163,8 @@ static int makeInfo(Agraph_t * graph)
 /* sort sites on y, then x, coord */
 static int scomp(const void *S1, const void *S2)
 {
-    const Site *s1, *s2;
-
-    s1 = *(Site *const *) S1;
-    s2 = *(Site *const *) S2;
+    const Site *s1 = *(Site *const *)S1;
+    const Site *s2 = *(Site *const *)S2;
     if (s1->coord.y < s2->coord.y)
 	return -1;
     if (s1->coord.y > s2->coord.y)
@@ -190,77 +176,61 @@ static int scomp(const void *S1, const void *S2)
     return 0;
 }
 
- /* sortSites:
-  * Fill array of pointer to sites and sort the sites using scomp
-  */
-static void sortSites(void)
-{
-    if (sites == 0) {
-	sites = gv_calloc(nsites, sizeof(Site*));
-	endSite = sites + nsites;
+ /// Fill array of pointer to sites and sort the sites using scomp
+static void sortSites(state_t *st) {
+    if (st->sites == NULL) {
+	st->sites = gv_calloc(nsites, sizeof(Site*));
+	st->endSite = st->sites + nsites;
     }
 
-    infoinit();
     for (size_t i = 0; i < nsites; i++) {
 	Info_t *ip = &nodeInfo[i];
-	sites[i] = &ip->site;
+	st->sites[i] = &ip->site;
 	ip->verts = NULL;
+	ip->n_verts = 0;
 	ip->site.refcnt = 1;
     }
 
-    qsort(sites, nsites, sizeof(Site *), scomp);
+    qsort(st->sites, nsites, sizeof(Site *), scomp);
 
     /* Reset site index for nextOne */
-    nextSite = sites;
+    st->nextSite = st->sites;
 
 }
 
-static void geomUpdate(int doSort)
-{
+static void geomUpdate(state_t *st, int doSort) {
     if (doSort)
-	sortSites();
+	sortSites(st);
 
     /* compute ranges */
     xmin = DBL_MAX;
     xmax = -DBL_MAX;
     assert(nsites > 0);
     for (size_t i = 0; i < nsites; ++i) {
-	xmin = fmin(xmin, sites[i]->coord.x);
-	xmax = fmax(xmax, sites[i]->coord.x);
+	xmin = fmin(xmin, st->sites[i]->coord.x);
+	xmax = fmax(xmax, st->sites[i]->coord.x);
     }
-    ymin = sites[0]->coord.y;
-    ymax = sites[nsites - 1]->coord.y;
+    ymin = st->sites[0]->coord.y;
+    ymax = st->sites[nsites - 1]->coord.y;
 
-    deltay = ymax - ymin;
     deltax = xmax - xmin;
 }
 
-static Site *nextOne(void)
-{
-    if (nextSite < endSite) {
-	return *nextSite++;
+static Site *nextOne(void *state) {
+    state_t *st = state;
+    if (st->nextSite < st->endSite) {
+	return *st->nextSite++;
     } else
 	return NULL;
 }
 
-/* rmEquality:
- * Check for nodes with identical positions and tweak
- * the positions.
- */
-static void rmEquality(void)
-{
-    int i, cnt;
-    Site **ip;
-    Site **jp;
-    Site **kp;
-    double xdel;
+/// Check for nodes with identical positions and tweak the positions.
+static void rmEquality(state_t *st) {
+    sortSites(st);
 
-    sortSites();
-    ip = sites;
-
-    while (ip < endSite) {
-	jp = ip + 1;
-	if (jp >= endSite ||
+    for (Site **ip = st->sites; ip < st->endSite; ) {
+	Site **jp = ip + 1;
+	if (jp >= st->endSite ||
 	    (*jp)->coord.x != (*ip)->coord.x ||
 	    (*jp)->coord.y != (*ip)->coord.y) {
 	    ip = jp;
@@ -268,9 +238,9 @@ static void rmEquality(void)
 	}
 
 	/* Find first node kp with position different from ip */
-	cnt = 2;
-	kp = jp + 1;
-	while (kp < endSite &&
+	int cnt = 2;
+	Site **kp = jp + 1;
+	while (kp < st->endSite &&
 	       (*kp)->coord.x == (*ip)->coord.x &&
 	       (*kp)->coord.y == (*ip)->coord.y) {
 	    cnt++;
@@ -279,9 +249,9 @@ static void rmEquality(void)
 	}
 
 	/* If next node exists and is on the same line */
-	if (kp < endSite && (*kp)->coord.y == (*ip)->coord.y) {
-	    xdel = ((*kp)->coord.x - (*ip)->coord.x) / cnt;
-	    i = 1;
+	if (kp < st->endSite && (*kp)->coord.y == (*ip)->coord.y) {
+	    const double xdel = ((*kp)->coord.x - (*ip)->coord.x) / cnt;
+	    int i = 1;
 	    for (jp = ip + 1; jp < kp; jp++) {
 		(*jp)->coord.x += i * xdel;
 		i++;
@@ -290,7 +260,7 @@ static void rmEquality(void)
 	    Info_t *info;
 	    for (jp = ip + 1; jp < kp; ip++, jp++) {
 		info = nodeInfo + (*ip)->sitenbr;
-		xdel = info->poly.corner.x - info->poly.origin.x;
+		double xdel = info->poly.corner.x - info->poly.origin.x;
 		info = nodeInfo + (*jp)->sitenbr;
 		xdel += info->poly.corner.x - info->poly.origin.x;
 		(*jp)->coord.x = (*ip)->coord.x + xdel / 2;
@@ -300,15 +270,12 @@ static void rmEquality(void)
     }
 }
 
-/* countOverlap:
- * Count number of node-node overlaps at iteration iter.
- */
-static int countOverlap(int iter)
-{
-    int count = 0;
+/// Count number of node-node overlaps at iteration iter.
+static unsigned countOverlap(unsigned iter) {
+    unsigned count = 0;
 
     for (size_t i = 0; i < nsites; i++)
-	nodeInfo[i].overlaps = 0;
+	nodeInfo[i].overlaps = false;
 
     for (size_t i = 0; i < nsites - 1; i++) {
 	Info_t *ip = &nodeInfo[i];
@@ -316,147 +283,129 @@ static int countOverlap(int iter)
 	    Info_t *jp = &nodeInfo[j];
 	    if (polyOverlap(ip->site.coord, &ip->poly, jp->site.coord, &jp->poly)) {
 		count++;
-		ip->overlaps = 1;
-		jp->overlaps = 1;
+		ip->overlaps = true;
+		jp->overlaps = true;
 	    }
 	}
     }
 
     if (Verbose > 1)
-	fprintf(stderr, "overlap [%d] : %d\n", iter, count);
+	fprintf(stderr, "overlap [%u] : %u\n", iter, count);
     return count;
 }
 
-static void increaseBoundBox(void)
-{
-    double ydelta, xdelta;
-    Point ll, ur;
+static void increaseBoundBox(state_t *st) {
+    Point ur = {.x = pxmax, .y = pymax};
+    Point ll = {.x = pxmin, .y = pymin};
 
-    ur.x = pxmax;
-    ur.y = pymax;
-    ll.x = pxmin;
-    ll.y = pymin;
-
-    ydelta = incr * (ur.y - ll.y);
-    xdelta = incr * (ur.x - ll.x);
+    const double ydelta = incr * (ur.y - ll.y);
+    const double xdelta = incr * (ur.x - ll.x);
 
     ur.x += xdelta;
     ur.y += ydelta;
     ll.x -= xdelta;
     ll.y -= ydelta;
 
-    setBoundBox(&ll, &ur);
+    setBoundBox(st, &ll, &ur);
 }
 
- /* areaOf:
-  * Area of triangle whose vertices are a,b,c
-  */
+/// Area of triangle whose vertices are a,b,c
 static double areaOf(Point a, Point b, Point c)
 {
     return fabs(a.x * (b.y - c.y) + b.x * (c.y - a.y) + c.x * (a.y - b.y)) / 2;
 }
 
- /* centroidOf:
-  * Compute centroid of triangle with vertices a, b, c.
-  * Return coordinates in x and y.
-  */
+/* Compute centroid of triangle with vertices a, b, c.
+ * Return coordinates in x and y.
+ */
 static void centroidOf(Point a, Point b, Point c, double *x, double *y)
 {
     *x = (a.x + b.x + c.x) / 3;
     *y = (a.y + b.y + c.y) / 3;
 }
 
- /* newpos;
-  * The new position is the centroid of the
-  * voronoi polygon. This is the weighted sum of the
-  * centroids of a triangulation, normalized to the
-  * total area.
-  */
+/* The new position is the centroid of the voronoi polygon. This is the weighted
+ * sum of the centroids of a triangulation, normalized to the total area.
+ */
 static void newpos(Info_t * ip)
 {
-    PtItem *anchor = ip->verts;
-    PtItem *p, *q;
+    const Point anchor = ip->verts[0];
     double totalArea = 0.0;
     double cx = 0.0;
     double cy = 0.0;
     double x;
     double y;
-    double area;
 
-    p = anchor->next;
-    q = p->next;
-    while (q != NULL) {
-	area = areaOf(anchor->p, p->p, q->p);
-	centroidOf(anchor->p, p->p, q->p, &x, &y);
+    for (size_t i = 1; i + 1 < ip->n_verts; ++i) {
+	const Point p = ip->verts[i];
+	const Point q = ip->verts[i + 1];
+	const double area = areaOf(anchor, p, q);
+	centroidOf(anchor, p, q, &x, &y);
 	cx += area * x;
 	cy += area * y;
 	totalArea += area;
-	p = q;
-	q = q->next;
     }
 
     ip->site.coord.x = cx / totalArea;
     ip->site.coord.y = cy / totalArea;
 }
 
- /* addCorners:
-  * Add corners of clipping window to appropriate sites.
+ /* Add corners of clipping window to appropriate sites.
   * A site gets a corner if it is the closest site to that corner.
   */
-static void addCorners(void)
-{
+static void addCorners(const state_t *st) {
     Info_t *ip = nodeInfo;
     Info_t *sws = ip;
     Info_t *nws = ip;
     Info_t *ses = ip;
     Info_t *nes = ip;
-    double swd = dist_2(&ip->site.coord, &sw);
-    double nwd = dist_2(&ip->site.coord, &nw);
-    double sed = dist_2(&ip->site.coord, &se);
-    double ned = dist_2(&ip->site.coord, &ne);
-    double d;
+    double swd = dist_2(ip->site.coord, st->sw);
+    double nwd = dist_2(ip->site.coord, st->nw);
+    double sed = dist_2(ip->site.coord, st->se);
+    double ned = dist_2(ip->site.coord, st->ne);
 
     for (size_t i = 1; i < nsites; i++) {
 	ip = &nodeInfo[i];
-	d = dist_2(&ip->site.coord, &sw);
+	double d = dist_2(ip->site.coord, st->sw);
 	if (d < swd) {
 	    swd = d;
 	    sws = ip;
 	}
-	d = dist_2(&ip->site.coord, &se);
+	d = dist_2(ip->site.coord, st->se);
 	if (d < sed) {
 	    sed = d;
 	    ses = ip;
 	}
-	d = dist_2(&ip->site.coord, &nw);
+	d = dist_2(ip->site.coord, st->nw);
 	if (d < nwd) {
 	    nwd = d;
 	    nws = ip;
 	}
-	d = dist_2(&ip->site.coord, &ne);
+	d = dist_2(ip->site.coord, st->ne);
 	if (d < ned) {
 	    ned = d;
 	    nes = ip;
 	}
     }
 
-    addVertex(&sws->site, sw.x, sw.y);
-    addVertex(&ses->site, se.x, se.y);
-    addVertex(&nws->site, nw.x, nw.y);
-    addVertex(&nes->site, ne.x, ne.y);
+    addVertex(&sws->site, st->sw.x, st->sw.y);
+    addVertex(&ses->site, st->se.x, st->se.y);
+    addVertex(&nws->site, st->nw.x, st->nw.y);
+    addVertex(&nes->site, st->ne.x, st->ne.y);
 }
 
- /* newPos:
-  * Calculate the new position of a site as the centroid
+ /* Calculate the new position of a site as the centroid
   * of its voronoi polygon, if it overlaps other nodes.
   * The polygons are finite by being clipped to the clipping
   * window.
   * We first add the corner of the clipping windows to the
   * vertex lists of the appropriate sites.
+  *
+  * @param st Algorithm state
+  * @param doAll Move all nodes, regardless of overlap
   */
-static void newPos(void)
-{
-    addCorners();
+static void newPos(const state_t *st, bool doAll) {
+    addCorners(st);
     for (size_t i = 0; i < nsites; i++) {
 	Info_t *ip = &nodeInfo[i];
 	if (doAll || ip->overlaps)
@@ -464,45 +413,37 @@ static void newPos(void)
     }
 }
 
-/* cleanup:
- * Cleanup voronoi memory.
- * Note that PQcleanup and ELcleanup rely on the number
+/* Cleanup voronoi memory.
+ * Note that ELcleanup relies on the number
  * of sites, so should at least be reset every time we use vAdjust.
  * This could be optimized, over multiple components or
  * even multiple graphs, but probably not worth it.
  */
 static void cleanup(void)
 {
-    PQcleanup();
     ELcleanup();
     siteinit();			/* free memory */
     edgeinit();			/* free memory */
 }
 
-static int vAdjust(void)
-{
-    int iterCnt = 0;
-    int overlapCnt = 0;
-    int badLevel = 0;
-    int increaseCnt = 0;
-    int cnt;
+static int vAdjust(state_t *st) {
+    unsigned iterCnt = 0;
+    unsigned badLevel = 0;
+    unsigned increaseCnt = 0;
 
-    if (!useIter || iterations > 0)
-	overlapCnt = countOverlap(iterCnt);
+    unsigned overlapCnt = countOverlap(iterCnt);
 
-    if (overlapCnt == 0 || iterations == 0)
+    if (overlapCnt == 0)
 	return 0;
 
-    rmEquality();
-    geomUpdate(0);
-    voronoi(nextOne);
-    while (1) {
-	newPos();
+    rmEquality(st);
+    geomUpdate(st, 0);
+    voronoi(nextOne, st);
+    for (bool doAll = false;;) {
+	newPos(st, doAll);
 	iterCnt++;
 
-	if (useIter && iterCnt == iterations)
-	    break;
-	cnt = countOverlap(iterCnt);
+	const unsigned cnt = countOverlap(iterCnt);
 	if (cnt == 0)
 	    break;
 	if (cnt >= overlapCnt)
@@ -518,25 +459,24 @@ static int vAdjust(void)
 	default:
 	    doAll = true;
 	    increaseCnt++;
-	    increaseBoundBox();
+	    increaseBoundBox(st);
 	    break;
 	}
 
-	geomUpdate(1);
-	voronoi(nextOne);
+	geomUpdate(st, 1);
+	voronoi(nextOne, st);
     }
 
     if (Verbose) {
-	fprintf(stderr, "Number of iterations = %d\n", iterCnt);
-	fprintf(stderr, "Number of increases = %d\n", increaseCnt);
+	fprintf(stderr, "Number of iterations = %u\n", iterCnt);
+	fprintf(stderr, "Number of increases = %u\n", increaseCnt);
     }
 
     cleanup();
     return 1;
 }
 
-static double rePos(void)
-{
+static void rePos(void) {
     double f = 1.0 + incr;
 
     for (size_t i = 0; i < nsites; i++) {
@@ -544,43 +484,34 @@ static double rePos(void)
 	ip->site.coord.x *= f;
 	ip->site.coord.y *= f;
     }
-    return f;
 }
 
-static int sAdjust(void)
-{
-    int iterCnt = 0;
-    int overlapCnt = 0;
-    int cnt;
+static int sAdjust(state_t *st) {
+    unsigned iterCnt = 0;
 
-    if (!useIter || iterations > 0)
-	overlapCnt = countOverlap(iterCnt);
+    const unsigned overlapCnt = countOverlap(iterCnt);
 
-    if (overlapCnt == 0 || iterations == 0)
+    if (overlapCnt == 0)
 	return 0;
 
-    rmEquality();
+    rmEquality(st);
     while (1) {
 	rePos();
 	iterCnt++;
 
-	if (useIter && iterCnt == iterations)
-	    break;
-	cnt = countOverlap(iterCnt);
+	const unsigned cnt = countOverlap(iterCnt);
 	if (cnt == 0)
 	    break;
     }
 
     if (Verbose) {
-	fprintf(stderr, "Number of iterations = %d\n", iterCnt);
+	fprintf(stderr, "Number of iterations = %u\n", iterCnt);
     }
 
     return 1;
 }
 
- /* updateGraph:
-  * Enter new node positions into the graph
-  */
+/// Enter new node positions into the graph
 static void updateGraph(void)
 {
     for (size_t i = 0; i < nsites; i++) {
@@ -591,23 +522,19 @@ static void updateGraph(void)
 }
 
 #define ELS "|edgelabel|"
-#define ELSN (sizeof(ELS)-1)
   /* Return true if node name starts with ELS */
-#define IS_LNODE(n) (!strncmp(agnameof(n),ELS,ELSN))
+#define IS_LNODE(n) startswith(agnameof(n), ELS)
 
-/* getSizes:
- * Set up array of half sizes in inches.
- */
+/// Set up array of half sizes in inches.
 double *getSizes(Agraph_t * g, pointf pad, int* n_elabels, int** elabels)
 {
-    Agnode_t *n;
     double *sizes = gv_calloc(Ndim * agnnodes(g), sizeof(double));
-    int i, nedge_nodes = 0;
+    int nedge_nodes = 0;
 
-    for (n = agfstnode(g); n; n = agnxtnode(g, n)) {
+    for (Agnode_t *n = agfstnode(g); n; n = agnxtnode(g, n)) {
 	if (elabels && IS_LNODE(n)) nedge_nodes++;
 
-	i = ND_id(n);
+	const int i = ND_id(n);
 	sizes[i * Ndim] = ND_width(n) * .5 + pad.x;
 	sizes[i * Ndim + 1] = ND_height(n) * .5 + pad.y;
     }
@@ -615,7 +542,7 @@ double *getSizes(Agraph_t * g, pointf pad, int* n_elabels, int** elabels)
     if (elabels && nedge_nodes) {
 	int* elabs = gv_calloc(nedge_nodes, sizeof(int));
 	nedge_nodes = 0;
-	for (n = agfstnode(g); n; n = agnxtnode(g, n)) {
+	for (Agnode_t *n = agfstnode(g); n; n = agnxtnode(g, n)) {
 	    if (IS_LNODE(n))
 		elabs[nedge_nodes++] = ND_id(n);
 	}
@@ -626,83 +553,58 @@ double *getSizes(Agraph_t * g, pointf pad, int* n_elabels, int** elabels)
     return sizes;
 }
 
-/* makeMatrix:
- * Assumes g is connected and simple, i.e., we can have a->b and b->a
+/* Assumes g is connected and simple, i.e., we can have a->b and b->a
  * but not a->b and a->b
  */
-SparseMatrix makeMatrix(Agraph_t* g, SparseMatrix *D)
-{
-    SparseMatrix A = 0;
-    Agnode_t *n;
-    Agedge_t *e;
-    Agsym_t *sym;
-    int nnodes;
-    int nedges;
-    int i, row;
-    double v;
-    int type = MATRIX_TYPE_REAL;
-    Agsym_t* symD = NULL;
-    double* valD = NULL;
-
+SparseMatrix makeMatrix(Agraph_t *g) {
     if (!g)
 	return NULL;
-    nnodes = agnnodes(g);
-    nedges = agnedges(g);
+    const int nnodes = agnnodes(g);
+    const int nedges = agnedges(g);
 
     /* Assign node ids */
-    i = 0;
-    for (n = agfstnode(g); n; n = agnxtnode(g, n))
+    int i = 0;
+    for (Agnode_t *n = agfstnode(g); n; n = agnxtnode(g, n))
 	ND_id(n) = i++;
 
     int *I = gv_calloc(nedges, sizeof(int));
     int *J = gv_calloc(nedges, sizeof(int));
     double *val = gv_calloc(nedges, sizeof(double));
 
-    sym = agfindedgeattr(g, "weight");
-    if (D) {
-	symD = agfindedgeattr(g, "len");
-	valD = gv_calloc(nedges, sizeof(double));
-    }
+    Agsym_t *sym = agfindedgeattr(g, "weight");
 
     i = 0;
-    for (n = agfstnode(g); n; n = agnxtnode(g, n)) {
-	row = ND_id(n);
-	for (e = agfstout(g, n); e; e = agnxtout(g, e)) {
+    for (Agnode_t *n = agfstnode(g); n; n = agnxtnode(g, n)) {
+	const int row = ND_id(n);
+	for (Agedge_t *e = agfstout(g, n); e; e = agnxtout(g, e)) {
 	    I[i] = row;
 	    J[i] = ND_id(aghead(e));
+	    double v;
 	    if (!sym || sscanf(agxget(e, sym), "%lf", &v) != 1)
 		v = 1;
 	    val[i] = v;
 	/* edge length */
-	    if (symD) {
-		if (sscanf (agxget (e, symD), "%lf", &v) != 1) v = 1;
-		valD[i] = v;
-	    }
 	    i++;
 	}
     }
 
-    A = SparseMatrix_from_coordinate_arrays(nedges, nnodes, nnodes, I, J,
-					    val, type, sizeof(double));
-
-    if (D) *D = SparseMatrix_from_coordinate_arrays(nedges, nnodes, nnodes, I, J, valD, type, sizeof(double));
+    SparseMatrix A = SparseMatrix_from_coordinate_arrays(nedges, nnodes, nnodes,
+                                                         I, J, val,
+                                                         MATRIX_TYPE_REAL,
+                                                         sizeof(double));
 
     free(I);
     free(J);
     free(val);
-    free (valD);
 
     return A;
 }
 
 #if ((defined(HAVE_GTS) || defined(HAVE_TRIANGLE)) && defined(SFDP))
 static void fdpAdjust(graph_t *g, adjust_data *am) {
-    SparseMatrix A0 = makeMatrix(g, NULL);
+    SparseMatrix A0 = makeMatrix(g);
     SparseMatrix A = A0;
-    double *sizes;
     double *pos = gv_calloc(Ndim * agnnodes(g), sizeof(double));
-    Agnode_t *n;
-    int i;
     expand_t sep = sepFactor(g);
     pointf pad;
 
@@ -713,11 +615,11 @@ static void fdpAdjust(graph_t *g, adjust_data *am) {
 	pad.x = PS2INCH(DFLT_MARGIN);
 	pad.y = PS2INCH(DFLT_MARGIN);
     }
-    sizes = getSizes(g, pad, NULL, NULL);
+    double *sizes = getSizes(g, pad, NULL, NULL);
 
-    for (n = agfstnode(g); n; n = agnxtnode(g, n)) {
+    for (Agnode_t *n = agfstnode(g); n; n = agnxtnode(g, n)) {
 	double* npos = pos + Ndim * ND_id(n);
-	for (i = 0; i < Ndim; i++) {
+	for (int i = 0; i < Ndim; i++) {
 	    npos[i] = ND_pos(n)[i];
 	}
     }
@@ -730,11 +632,11 @@ static void fdpAdjust(graph_t *g, adjust_data *am) {
 
     remove_overlap(Ndim, A, pos, sizes, am->value, am->scaling, 
                    ELSCHEME_NONE, 0, NULL, NULL,
-                   mapBool(agget(g, "overlap_shrink"), true) ? TRUE : FALSE);
+                   mapBool(agget(g, "overlap_shrink"), true));
 
-    for (n = agfstnode(g); n; n = agnxtnode(g, n)) {
+    for (Agnode_t *n = agfstnode(g); n; n = agnxtnode(g, n)) {
 	double *npos = pos + Ndim * ND_id(n);
-	for (i = 0; i < Ndim; i++) {
+	for (int i = 0; i < Ndim; i++) {
 	    ND_pos(n)[i] = npos[i];
 	}
     }
@@ -757,17 +659,14 @@ vpscAdjust(graph_t* G)
     pointf *nsize = gv_calloc(nnodes, sizeof(pointf));
     float* coords[dim];
     float *f_storage = gv_calloc(dim * nnodes, sizeof(float));
-    int i, j;
-    Agnode_t* v;
-    expand_t exp_margin;
 
-    for (i = 0; i < dim; i++) {
+    for (size_t i = 0; i < dim; i++) {
 	coords[i] = f_storage + i * nnodes;
     }
 
-    j = 0;
-    for (v = agfstnode(G); v; v = agnxtnode(G, v)) {
-	for (i = 0; i < dim; i++) {
+    size_t j = 0;
+    for (Agnode_t *v = agfstnode(G); v; v = agnxtnode(G, v)) {
+	for (size_t i = 0; i < dim; i++) {
 	    coords[i][j] =  (float)ND_pos(v)[i];
 	}
 	nsize[j].x = ND_width(v);
@@ -778,8 +677,8 @@ vpscAdjust(graph_t* G)
     opt.diredges = 0;
     opt.edge_gap = 0;
     opt.noverlap = 2;
-    opt.clusters = gv_alloc(sizeof(cluster_data));
-    exp_margin = sepFactor (G);
+    opt.clusters = (cluster_data){0};
+    expand_t exp_margin = sepFactor (G);
  	/* Multiply by 2 since opt.gap is the gap size, not the margin */
     if (exp_margin.doAdd) {
 	opt.gap.x = 2.0*PS2INCH(exp_margin.x);
@@ -793,35 +692,32 @@ vpscAdjust(graph_t* G)
     removeoverlaps(nnodes, coords, &opt);
 
     j = 0;
-    for (v = agfstnode(G); v; v = agnxtnode(G, v)) {
-	for (i = 0; i < dim; i++) {
+    for (Agnode_t *v = agfstnode(G); v; v = agnxtnode(G, v)) {
+	for (size_t i = 0; i < dim; i++) {
 	    ND_pos(v)[i] = coords[i][j];
 	}
 	j++;
     }
 
-    free (opt.clusters);
     free (f_storage);
     free (nsize);
     return 0;
 }
 #endif
 
-/* angleSet:
- * Return true if "normalize" is defined and valid; return angle in phi.
+/* Return true if "normalize" is defined and valid; return angle in phi.
  * Read angle as degrees, convert to radians.
  * Guarantee -PI < phi <= PI.
  */
 static int
 angleSet (graph_t* g, double* phi)
 {
-    double ang;
     char* p;
     char* a = agget(g, "normalize");
 
     if (!a || *a == '\0')
 	return 0;
-    ang = strtod (a, &p);
+    double ang = strtod (a, &p);
     if (p == a) {  /* no number */
 	if (mapbool(a))
 	    ang = 0.0;
@@ -835,27 +731,21 @@ angleSet (graph_t* g, double* phi)
     return 1;
 }
 
-/* normalize:
- * If normalize is set, move first node to origin, then
+/* If normalize is set, move first node to origin, then
  * rotate graph so that the angle of the first edge is given
  * by the degrees from normalize.
  * FIX: Generalize to allow rotation determined by graph shape.
  */
 int normalize(graph_t * g)
 {
-    node_t *v;
-    edge_t *e;
     double phi;
-    double cosv, sinv;
-    pointf p, orig;
     int ret;
 
     if (!angleSet(g, &phi))
 	return 0;
 
-    v = agfstnode(g);
-    p.x = ND_pos(v)[0];
-    p.y = ND_pos(v)[1];
+    node_t *v = agfstnode(g);
+    pointf p = {.x = ND_pos(v)[0], .y = ND_pos(v)[1]};
     for (v = agfstnode(g); v; v = agnxtnode(g, v)) {
 	ND_pos(v)[0] -= p.x;
 	ND_pos(v)[1] -= p.y;
@@ -863,7 +753,7 @@ int normalize(graph_t * g)
     if (p.x || p.y) ret = 1;
     else ret = 0;
 
-    e = NULL;
+    edge_t *e = NULL;
     for (v = agfstnode(g); v; v = agnxtnode(g, v))
 	if ((e = agfstout(g, v)))
 	    break;
@@ -875,10 +765,10 @@ int normalize(graph_t * g)
 		   ND_pos(aghead(e))[0] - ND_pos(agtail(e))[0]);
 
     if (phi) {
-	orig.x = ND_pos(agtail(e))[0];
-	orig.y = ND_pos(agtail(e))[1];
-	cosv = cos(phi);
-	sinv = sin(phi);
+	const pointf orig = {.x = ND_pos(agtail(e))[0],
+	                     .y = ND_pos(agtail(e))[1]};
+	const double cosv = cos(phi);
+	const double sinv = sin(phi);
 	for (v = agfstnode(g); v; v = agnxtnode(g, v)) {
 	    p.x = ND_pos(v)[0] - orig.x;
 	    p.y = ND_pos(v)[1] - orig.y;
@@ -893,50 +783,41 @@ int normalize(graph_t * g)
 typedef struct {
     adjust_mode mode;
     char *attrib;
-    int len;
     char *print;
 } lookup_t;
 
-#define STRLEN(s) ((sizeof(s)-1)/sizeof(char))
-#define ITEM(i,s,v) {i, s, STRLEN(s), v}
-
 /* Translation table from overlap values to algorithms.
  * adjustMode[0] corresponds to overlap=true
  * adjustMode[1] corresponds to overlap=false
  */
 static const lookup_t adjustMode[] = {
-    ITEM(AM_NONE, "", "none"),
+    {AM_NONE, "", "none"},
 #if ((defined(HAVE_GTS) || defined(HAVE_TRIANGLE)) && defined(SFDP))
-    ITEM(AM_PRISM, "prism", "prism"),
+    {AM_PRISM, "prism", "prism"},
 #endif
-    ITEM(AM_VOR, "voronoi", "Voronoi"),
-    ITEM(AM_NSCALE, "scale", "scaling"),
-    ITEM(AM_COMPRESS, "compress", "compress"),
-    ITEM(AM_VPSC, "vpsc", "vpsc"),
-    ITEM(AM_IPSEP, "ipsep", "ipsep"),
-    ITEM(AM_SCALE, "oscale", "old scaling"),
-    ITEM(AM_SCALEXY, "scalexy", "x and y scaling"),
-    ITEM(AM_ORTHO, "ortho", "orthogonal constraints"),
-    ITEM(AM_ORTHO_YX, "ortho_yx", "orthogonal constraints"),
-    ITEM(AM_ORTHOXY, "orthoxy", "xy orthogonal constraints"),
-    ITEM(AM_ORTHOYX, "orthoyx", "yx orthogonal constraints"),
-    ITEM(AM_PORTHO, "portho", "pseudo-orthogonal constraints"),
-    ITEM(AM_PORTHO_YX, "portho_yx", "pseudo-orthogonal constraints"),
-    ITEM(AM_PORTHOXY, "porthoxy", "xy pseudo-orthogonal constraints"),
-    ITEM(AM_PORTHOYX, "porthoyx", "yx pseudo-orthogonal constraints"),
+    {AM_VOR, "voronoi", "Voronoi"},
+    {AM_NSCALE, "scale", "scaling"},
+    {AM_COMPRESS, "compress", "compress"},
+    {AM_VPSC, "vpsc", "vpsc"},
+    {AM_IPSEP, "ipsep", "ipsep"},
+    {AM_SCALE, "oscale", "old scaling"},
+    {AM_SCALEXY, "scalexy", "x and y scaling"},
+    {AM_ORTHO, "ortho", "orthogonal constraints"},
+    {AM_ORTHO_YX, "ortho_yx", "orthogonal constraints"},
+    {AM_ORTHOXY, "orthoxy", "xy orthogonal constraints"},
+    {AM_ORTHOYX, "orthoyx", "yx orthogonal constraints"},
+    {AM_PORTHO, "portho", "pseudo-orthogonal constraints"},
+    {AM_PORTHO_YX, "portho_yx", "pseudo-orthogonal constraints"},
+    {AM_PORTHOXY, "porthoxy", "xy pseudo-orthogonal constraints"},
+    {AM_PORTHOYX, "porthoyx", "yx pseudo-orthogonal constraints"},
 #if !((defined(HAVE_GTS) || defined(HAVE_TRIANGLE)) && defined(SFDP))
-    ITEM(AM_PRISM, "prism", 0),
+    {AM_PRISM, "prism", 0},
 #endif
-    {AM_NONE, 0, 0, 0}
+    {0}
 };
 
-    
-/* setPrismValues:
- * Initialize and set prism values
- */
-static void
-setPrismValues (Agraph_t* g, char* s, adjust_data* dp)
-{
+/// Initialize and set prism values
+static void setPrismValues(Agraph_t *g, const char *s, adjust_data *dp) {
     int v;
 
     if (sscanf (s, "%d", &v) > 0 && v >= 0)
@@ -946,12 +827,8 @@ setPrismValues (Agraph_t* g, char* s, adjust_data* dp)
     dp->scaling = late_double(g, agfindgraphattr(g, "overlap_scaling"), -4.0, -1.e10);
 }
 
-/* getAdjustMode:
- * Convert string value to internal value of adjustment mode.
- * If s is NULL or empty, return NONE.
- */
-static adjust_data *getAdjustMode(Agraph_t* g, char *s, adjust_data* dp)
-{
+/// Convert string value to internal value of adjustment mode.
+static void getAdjustMode(Agraph_t *g, const char *s, adjust_data *dp) {
     const lookup_t *ap = adjustMode + 1;
     if (s == NULL || *s == '\0') {
 	dp->mode = adjustMode[0].mode;
@@ -959,24 +836,28 @@ static adjust_data *getAdjustMode(Agraph_t* g, char *s, adjust_data* dp)
     }
     else {
 	while (ap->attrib) {
-	    if (!strncasecmp(s, ap->attrib, ap->len)) {
+	    bool matches = strcasecmp(s, ap->attrib) == 0;
+	    // "prism" takes parameters, so needs to match "prism.*"
+	    matches |= ap->mode == AM_PRISM
+	            && strncasecmp(s, ap->attrib, strlen(ap->attrib)) == 0;
+	    if (matches) {
 		if (ap->print == NULL) {
-		    agerr (AGWARN, "Overlap value \"%s\" unsupported - ignored\n", ap->attrib);
+		    agwarningf("Overlap value \"%s\" unsupported - ignored\n", ap->attrib);
 		    ap = &adjustMode[1];
 		}
 		dp->mode = ap->mode;
 		dp->print = ap->print;
 		if (ap->mode == AM_PRISM)
-		    setPrismValues (g, s + ap->len, dp);
+		    setPrismValues(g, s + strlen(ap->attrib), dp);
 		break;
 	    }
 	    ap++;
 	}
 	if (ap->attrib == NULL ) {
-	    bool v = mapBool(s, false);
+	    bool v = mapbool(s);
 	    bool unmappable = v != mapBool(s, true);
 	    if (unmappable) {
-		agerr (AGWARN, "Unrecognized overlap value \"%s\" - using false\n", s);
+		agwarningf("Unrecognized overlap value \"%s\" - using false\n", s);
 		v = false;
 	    }
 	    if (v) {
@@ -994,23 +875,18 @@ static adjust_data *getAdjustMode(Agraph_t* g, char *s, adjust_data* dp)
     if (Verbose) {
 	fprintf(stderr, "overlap: %s value %d scaling %.04f\n", dp->print, dp->value, dp->scaling);
     }
-    return dp;
 }
 
-adjust_data *graphAdjustMode(graph_t *G, adjust_data* dp, char* dflt)
-{
+void graphAdjustMode(graph_t *G, adjust_data *dp, char *dflt) {
     char* am = agget(G, "overlap");
-    return getAdjustMode (G, am ? am : (dflt ? dflt : ""), dp);
+    getAdjustMode (G, am ? am : (dflt ? dflt : ""), dp);
 }
 
 #define ISZERO(d) (fabs(d) < 0.000000001)
 
-/* simpleScaling:
- */
 static int simpleScale (graph_t* g) 
 {
     pointf sc;
-    node_t* n;
     int i;
     char* p;
 
@@ -1022,7 +898,7 @@ static int simpleScale (graph_t* g)
 	    if (sc.y == 1 && sc.x == 1) return 0;
 	    if (Verbose)
 		fprintf (stderr, "scale = (%.03f,%.03f)\n", sc.x, sc.y);
-	    for (n = agfstnode(g); n; n = agnxtnode(g,n)) {
+	    for (node_t *n = agfstnode(g); n; n = agnxtnode(g,n)) {
 		ND_pos(n)[0] *= sc.x;
 		ND_pos(n)[1] *= sc.y;
 	    }
@@ -1032,20 +908,19 @@ static int simpleScale (graph_t* g)
     return 0;
 }
 
-/* removeOverlapWith:
- * Use adjust_data to determine if and how to remove
+/* Use adjust_data to determine if and how to remove
  * node overlaps.
  * Return non-zero if nodes are moved.
  */
 int 
 removeOverlapWith (graph_t * G, adjust_data* am)
 {
-    int ret, nret;
+    int ret;
 
     if (agnnodes(G) < 2)
 	return 0;
 
-    nret = normalize (G);
+    int nret = normalize (G);
     nret += simpleScale (G);
 
     if (am->mode == AM_NONE)
@@ -1098,7 +973,7 @@ removeOverlapWith (graph_t * G, adjust_data* am)
 #endif
 	default:		/* to silence warnings */
 	    if (am->mode != AM_VOR && am->mode != AM_SCALE)
-		agerr(AGWARN, "Unhandled adjust option %s\n", am->print);
+		agwarningf("Unhandled adjust option %s\n", am->print);
 	    ret = 0;
 	    break;
 	}
@@ -1108,33 +983,28 @@ removeOverlapWith (graph_t * G, adjust_data* am)
     /* create main array */
     if (makeInfo(G)) {
 	freeNodes();
-	free(sites);
-	sites = 0;
 	return nret;
     }
 
     /* establish and verify bounding box */
-    chkBoundBox(G);
+    state_t st = {0};
+    chkBoundBox(&st, G);
 
     if (am->mode == AM_SCALE)
-	ret = sAdjust();
+	ret = sAdjust(&st);
     else
-	ret = vAdjust();
+	ret = vAdjust(&st);
 
     if (ret)
 	updateGraph();
 
     freeNodes();
-    free(sites);
-    sites = 0;
+    free(st.sites);
 
     return ret+nret;
 }
 
-/* removeOverlapAs:
- * Use flag value to determine if and how to remove
- * node overlaps.
- */
+/// Use flag value to determine if and how to remove node overlaps.
 int 
 removeOverlapAs(graph_t * G, char* flag)
 {
@@ -1146,8 +1016,7 @@ removeOverlapAs(graph_t * G, char* flag)
     return removeOverlapWith (G, &am);
 }
 
-/* adjustNodes:
- * Remove node overlap relying on graph's overlap attribute.
+/* Remove node overlap relying on graph's overlap attribute.
  * Return non-zero if graph has changed.
  */
 int adjustNodes(graph_t * G)
@@ -1155,35 +1024,32 @@ int adjustNodes(graph_t * G)
     return removeOverlapAs(G, agget(G, "overlap"));
 }
 
-/* parseFactor:
- * Convert "sep" attribute into expand_t.
+/* Convert "sep" attribute into expand_t.
  * Input "+x,y" becomes {x,y,true}
  * Input "x,y" becomes {1 + x/sepfact,1 + y/sepfact,false}
  * Return 1 on success, 0 on failure
  */
-static int
-parseFactor (char* s, expand_t* pp, float sepfact, float dflt)
-{
+static int parseFactor(char *s, expand_t *pp, double sepfact, double dflt) {
     int i;
-    float x, y;
 
-    while (isspace((int)*s)) s++;
+    while (gv_isspace(*s)) s++;
     if (*s == '+') {
 	s++;
 	pp->doAdd = true;
     }
     else pp->doAdd = false;
 
-    if ((i = sscanf(s, "%f,%f", &x, &y))) {
+    double x, y;
+    if ((i = sscanf(s, "%lf,%lf", &x, &y))) {
 	if (i == 1) y = x;
 	if (pp->doAdd) {
 	    if (sepfact > 1) {
-		pp->x = fminf(dflt, x/sepfact);
-		pp->y = fminf(dflt, y/sepfact);
+		pp->x = fmin(dflt, x / sepfact);
+		pp->y = fmin(dflt, y / sepfact);
 	    }
 	    else if (sepfact < 1) {
-		pp->x = fmaxf(dflt, x/sepfact);
-		pp->y = fmaxf(dflt, y/sepfact);
+		pp->x = fmax(dflt, x / sepfact);
+		pp->y = fmax(dflt, y / sepfact);
 	    }
 	    else {
 		pp->x = x;
@@ -1191,16 +1057,14 @@ parseFactor (char* s, expand_t* pp, float sepfact, float dflt)
 	    }
 	}
 	else {
-	    pp->x = 1.0f + x/sepfact;
-	    pp->y = 1.0f + y/sepfact;
+	    pp->x = 1.0 + x / sepfact;
+	    pp->y = 1.0 + y / sepfact;
 	}
 	return 1;
     }
     else return 0;
 }
 
-/* sepFactor:
- */
 expand_t
 sepFactor(graph_t* g)
 {
@@ -1221,8 +1085,7 @@ sepFactor(graph_t* g)
     return pmargin;
 }
 
-/* esepFactor:
- * This value should be smaller than the sep value used to expand
+/* This value should be smaller than the sep value used to expand
  * nodes during adjustment. If not, when the adjustment pass produces
  * a fairly tight layout, the spline code will find that some nodes
  * still overlap.
@@ -1235,7 +1098,8 @@ esepFactor(graph_t* g)
 
     if ((marg = agget(g, "esep")) && parseFactor(marg, &pmargin, 1.0, 0)) {
     }
-    else if ((marg = agget(g, "sep")) && parseFactor(marg, &pmargin, 1.0f/SEPFACT, SEPFACT*DFLT_MARGIN)) {
+    else if ((marg = agget(g, "sep")) &&
+             parseFactor(marg, &pmargin, 1.0 / SEPFACT, SEPFACT * DFLT_MARGIN)) {
     }
     else {
 	pmargin.x = pmargin.y = SEPFACT*DFLT_MARGIN;
