@@ -34,16 +34,16 @@
 #include <common/utils.h>
 #include <neatogen/sgd.h>
 #include <cgraph/agxbuf.h>
-#include <cgraph/alloc.h>
 #include <cgraph/bitarray.h>
 #include <cgraph/cgraph.h>
 #include <cgraph/gv_ctype.h>
-#include <cgraph/prisize_t.h>
 #include <cgraph/startswith.h>
-#include <cgraph/strcasecmp.h>
 #include <float.h>
 #include <stdbool.h>
 #include <stddef.h>
+#include <util/alloc.h>
+#include <util/prisize_t.h>
+#include <util/strcasecmp.h>
 #include <util/streq.h>
 
 #ifndef HAVE_SRAND48
