@@ -3,7 +3,7 @@
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
@@ -23,24 +23,36 @@
 
 #define DEBUG
 #include <assert.h>
+#include <float.h>
+#include <math.h>
 #include <stdbool.h>
 #include <stddef.h>
 #include <ortho/maze.h>
 #include <ortho/fPQ.h>
 #include <ortho/ortho.h>
-#include <common/memory.h>
 #include <common/geomprocs.h>
 #include <common/globals.h>
 #include <common/render.h>
 #include <common/pointset.h>
+#include <util/alloc.h>
+#include <util/exit.h>
+#include <util/unused.h>
+
 typedef struct {
     int d;
     Agedge_t* e;
 } epair_t;
 
 #ifdef DEBUG
-static void emitSearchGraph (FILE* fp, sgraph* sg);
-static void emitGraph (FILE* fp, maze* mp, size_t n_edges, route* route_list, epair_t[]);
+#define DEBUG_FN /* nothing */
+#else
+#define DEBUG_FN UNUSED
+#endif
+
+static DEBUG_FN void emitSearchGraph(FILE *fp, sgraph *sg);
+static DEBUG_FN void emitGraph(FILE *fp, maze *mp, size_t n_edges,
+                               route *route_list, epair_t[]);
+#ifdef DEBUG
 int odb_flags;
 #endif
 
@@ -114,14 +126,12 @@ setSeg (segment* sp, bool dir, double fix, double b1, double b2, int l1, int l2)
 	sp->p.p2 = b2;
 	sp->l1 = l1;
 	sp->l2 = l2;
-	sp->flipped = 0;
     }
     else {
 	sp->p.p2 = b1;
 	sp->p.p1 = b2;
 	sp->l2 = l1;
 	sp->l1 = l2;
-	sp->flipped = 1;
     }
 }
 
@@ -132,8 +142,6 @@ setSeg (segment* sp, bool dir, double fix, double b1, double b2, int l1, int l2)
  * the two dummy nodes representing the center of the two real nodes,
  * and the two nodes on the boundary of the two real nodes.
  */
-#define PUSH(rte,P) (rte.p[rte.n++] = P)
-
 static route
 convertSPtoRoute (sgraph* g, snode* fst, snode* lst)
 {
@@ -153,7 +161,7 @@ convertSPtoRoute (sgraph* g, snode* fst, snode* lst)
     for (ptr = fst; ptr; ptr = N_DAD(ptr)) sz++;
     rte.n = 0;
     assert(sz >= 2);
-    rte.segs = N_NEW(sz-2, segment);  /* at most sz-2 segments */
+    rte.segs = gv_calloc(sz - 2, sizeof(segment));  /* at most sz-2 segments */
 
     seg.prev = seg.next = 0;
     ptr = prev = N_DAD(fst);
@@ -230,7 +238,7 @@ convertSPtoRoute (sgraph* g, snode* fst, snode* lst)
 	next = N_DAD(next);
     }
 
-    rte.segs = realloc (rte.segs, rte.n*sizeof(segment));
+    rte.segs = gv_recalloc(rte.segs, sz - 2, rte.n, sizeof(segment));
     for (size_t i=0; i<rte.n; i++) {
 	if (i > 0)
 	    rte.segs[i].prev = rte.segs + (i-1);
@@ -247,23 +255,15 @@ typedef struct {
     Dt_t*     chans;
 } chanItem;
 
-static void
-freeChannel (Dt_t* d, channel* cp, Dtdisc_t* disc)
-{
-    (void)d;
-    (void)disc;
-
+static void freeChannel(void *chan) {
+    channel *cp = chan;
     free_graph (cp->G);
-    free (cp->seg_list);
+    seg_list_free(&cp->seg_list);
     free (cp);
 }
 
-static void
-freeChanItem (Dt_t* d, chanItem* cp, Dtdisc_t* disc)
-{
-    (void)d;
-    (void)disc;
-
+static void freeChanItem(void *item) {
+    chanItem *cp = item;
     dtclose (cp->chans);
     free (cp);
 }
@@ -277,12 +277,9 @@ freeChanItem (Dt_t* d, chanItem* cp, Dtdisc_t* disc)
  * When searching for a channel containing a segment, we rely on
  * interval containment to return the correct channel.
  */
-static int
-chancmpid(Dt_t* d, paird* key1, paird* key2, Dtdisc_t* disc)
-{
-  (void)d;
-  (void)disc;
-
+static int chancmpid(void *k1, void *k2) {
+  const paird *key1 = k1;
+  const paird *key2 = k2;
   if (key1->p1 > key2->p1) {
     if (key1->p2 <= key2->p2) return 0;
     else return 1;
@@ -294,12 +291,9 @@ chancmpid(Dt_t* d, paird* key1, paird* key2, Dtdisc_t* disc)
   else return 0;
 }   
 
-static int
-dcmpid(Dt_t* d, double* key1, double* key2, Dtdisc_t* disc)
-{
-  (void)d;
-  (void)disc;
-
+static int dcmpid(void *k1, void *k2) {
+  const double *key1 = k1;
+  const double *key2 = k2;
   if (*key1 > *key2) return 1;
   else if (*key1 < *key2) return -1;
   else return 0;
@@ -310,11 +304,8 @@ static Dtdisc_t chanDisc = {
     sizeof(paird),
     offsetof(channel,link),
     0,
-    (Dtfree_f)freeChannel,
-    (Dtcompar_f)chancmpid,
-    0,
-    0,
-    0
+    freeChannel,
+    chancmpid,
 };
 
 static Dtdisc_t chanItemDisc = {
@@ -322,11 +313,8 @@ static Dtdisc_t chanItemDisc = {
     sizeof(double),
     offsetof(chanItem,link),
     0,
-    (Dtfree_f)freeChanItem,
-    (Dtcompar_f)dcmpid,
-    0,
-    0,
-    0
+    freeChanItem,
+    dcmpid,
 };
 
 static void
@@ -335,7 +323,7 @@ addChan (Dt_t* chdict, channel* cp, double j)
     chanItem* subd = dtmatch (chdict, &j);
 
     if (!subd) {
-	subd = NEW (chanItem);
+	subd = gv_alloc(sizeof(chanItem));
 	subd->v = j;
 	subd->chans = dtopen (&chanDisc, Dtoset);
 	dtinsert (chdict, subd);
@@ -362,7 +350,7 @@ extractHChans (maze* mp)
 	    cp = nextcp;
 	}
 
-	chp = NEW(channel);
+	chp = gv_alloc(sizeof(channel));
 	chp->cp = cp;
 	chp->p.p1 = cp->bb.LL.x;
 
@@ -399,7 +387,7 @@ extractVChans (maze* mp)
 	    cp = nextcp;
 	}
 
-	chp = NEW(channel);
+	chp = gv_alloc(sizeof(channel));
 	chp->cp = cp;
 	chp->p.p1 = cp->bb.LL.y;
 
@@ -420,9 +408,8 @@ extractVChans (maze* mp)
 static void
 insertChan (channel* chan, segment* seg)
 {
-    seg->ind_no = chan->cnt++;
-    chan->seg_list = ALLOC(chan->cnt, chan->seg_list, segment*);
-    chan->seg_list[chan->cnt-1] = seg;
+    seg->ind_no = seg_list_size(&chan->seg_list);
+    seg_list_append(&chan->seg_list, seg);
 }
 
 static channel*
@@ -502,8 +489,6 @@ addNodeEdges (sgraph* sg, cell* cp, snode* np)
 #endif
 }
 
-#ifdef DEBUG
-
 #include <common/intset.h>
 static char* bendToStr (bend b)
 {
@@ -539,28 +524,24 @@ static void putSeg (FILE* fp, segment* seg)
       seg->p.p2, seg->comm_coord, bendToStr (seg->l1), bendToStr (seg->l2));
 }
 
-static void
-dumpChanG (channel* cp, int v)
-{
-  int k;
+static DEBUG_FN void dumpChanG(channel *cp, double v) {
   intitem* ip;
   Dt_t* adj;
 
-  if (cp->cnt < 2) return;
-  fprintf (stderr, "channel %d (%f,%f)\n", v, cp->p.p1, cp->p.p2);
-  for (k=0;k<cp->cnt;k++) {
+  if (seg_list_size(&cp->seg_list) < 2) return;
+  fprintf (stderr, "channel %.0f (%f,%f)\n", v, cp->p.p1, cp->p.p2);
+  for (size_t k = 0; k < seg_list_size(&cp->seg_list); ++k) {
     adj = cp->G->vertices[k].adj_list;
     if (dtsize(adj) == 0) continue;
-    putSeg (stderr, cp->seg_list[k]);
+    putSeg(stderr, seg_list_get(&cp->seg_list, k));
     fputs (" ->\n", stderr);
     for (ip = dtfirst(adj); ip; ip = dtnext(adj, ip)) {
       fputs ("     ", stderr);
-      putSeg (stderr, cp->seg_list[ip->id]);
+      putSeg(stderr, seg_list_get(&cp->seg_list, ip->id));
       fputs ("\n", stderr);
     }
   }
 }
-#endif
 
 static void
 assignTrackNo (Dt_t* chans)
@@ -569,19 +550,18 @@ assignTrackNo (Dt_t* chans)
     Dtlink_t* l1;
     Dtlink_t* l2;
     channel* cp;
-    int k;
 
     for (l1 = dtflatten (chans); l1; l1 = dtlink(chans,l1)) {
 	lp = ((chanItem*)l1)->chans;
 	for (l2 = dtflatten (lp); l2; l2 = dtlink(lp,l2)) {
 	    cp = (channel*)l2;
-	    if (cp->cnt) {
+	    if (!seg_list_is_empty(&cp->seg_list)) {
 #ifdef DEBUG
     if (odb_flags & ODB_CHANG) dumpChanG (cp, ((chanItem*)l1)->v);
 #endif
 		top_sort (cp->G);
-		for (k=0;k<cp->cnt;k++)
-		    cp->seg_list[k]->track_no = cp->G->vertices[k].topsort_order+1;
+		for (size_t k = 0; k < seg_list_size(&cp->seg_list); ++k)
+		    seg_list_get(&cp->seg_list, k)->track_no = cp->G->vertices[k].topsort_order+1;
 	    }
    	}
     }
@@ -599,7 +579,7 @@ create_graphs(Dt_t* chans)
 	lp = ((chanItem*)l1)->chans;
 	for (l2 = dtflatten (lp); l2; l2 = dtlink(lp,l2)) {
 	    cp = (channel*)l2;
-	    cp->G = make_graph (cp->cnt);
+	    cp->G = make_graph(seg_list_size(&cp->seg_list));
    	}
     }
 }
@@ -621,15 +601,13 @@ overlapSeg (segment* S1, segment* S2, bend T1, bend T2)
 		else if(S1->l2==T2&&S2->l1==T1) return(1);
 		else return(0);
 	}
-	else if(S1->p.p2==S2->p.p2) {
-		if(S2->l1==T2) return eqEndSeg (S1->l2, S2->l2, T1, T2);
-		else return -1*eqEndSeg (S2->l2, S1->l2, T1, T2);
-	}
-	else { /* S1->p.p2>S2->p.p2 */
+	if (S1->p.p2 > S2->p.p2) {
 		if(S2->l1==T2&&S2->l2==T2) return(-1);
 		else if (S2->l1==T1&&S2->l2==T1) return(1);
 		else return(0);
 	}
+	if (S2->l1 == T2) return eqEndSeg (S1->l2, S2->l2, T1, T2);
+	return -1 * eqEndSeg(S2->l2, S1->l2, T1, T2);
 }
 
 static int
@@ -651,67 +629,60 @@ segCmp (segment* S1, segment* S2, bend T1, bend T2)
     if(S1->p.p1<S2->p.p1&&S2->p.p1<S1->p.p2)
 	return overlapSeg (S1, S2, T1, T2);
 	/* left endpoint of S1 inside S2 */
-    else if(S2->p.p1<S1->p.p1&&S1->p.p1<S2->p.p2)
+    if (S2->p.p1 < S1->p.p1 && S1->p.p1 < S2->p.p2)
 	return -1*overlapSeg (S2, S1, T1, T2);
-    else if(S1->p.p1==S2->p.p1) {
+    if (S1->p.p1 == S2->p.p1) {
 	if(S1->p.p2==S2->p.p2) {
 	    if (S1->l1 == S2->l1 && S1->l2 == S2->l2)
 		return 0;
-	    else if (S2->l1==S2->l2) {
+	    if (S2->l1 == S2->l2) {
 		if (S2->l1 == T1) return 1;
-		else if (S2->l1 == T2) return -1;
-		else if (S1->l1 != T1 && S1->l2 != T1) return 1;
-		else if (S1->l1 != T2 && S1->l2 != T2) return -1;
-		else return 0;
+		if (S2->l1 == T2) return -1;
+		if (S1->l1 != T1 && S1->l2 != T1) return 1;
+		if (S1->l1 != T2 && S1->l2 != T2) return -1;
+		return 0;
 	    }
-	    else if (S2->l1 == T1 && S2->l2 == T2) {
+	    if (S2->l1 == T1 && S2->l2 == T2) {
 		if (S1->l1 != T1 && S1->l2 == T2) return 1;
-		else if (S1->l1 == T1 && S1->l2 != T2) return -1;
-		else return 0;
+		if (S1->l1 == T1 && S1->l2 != T2) return -1;
+		return 0;
 	    }
-	    else if (S2->l2 == T1 && S2->l1 == T2) {
+	    if (S2->l2 == T1 && S2->l1 == T2) {
 		if (S1->l2 != T1 && S1->l1 == T2) return 1;
-		else if (S1->l2 == T1 && S1->l1 != T2) return -1;
-		else return 0;
+		if (S1->l2 == T1 && S1->l1 != T2) return -1;
+		return 0;
 	    }
-	    else if (S2->l1 == B_NODE && S2->l2 == T1) {
+	    if (S2->l1 == B_NODE && S2->l2 == T1) {
 		return ellSeg (S1->l1, S1->l2, T1);
 	    }
-	    else if (S2->l1 == B_NODE && S2->l2 == T2) {
+	    if (S2->l1 == B_NODE && S2->l2 == T2) {
 		return -1*ellSeg (S1->l1, S1->l2, T2);
 	    }
-	    else if (S2->l1 == T1 && S2->l2 == B_NODE) {
+	    if (S2->l1 == T1 && S2->l2 == B_NODE) {
 		return ellSeg (S1->l2, S1->l1, T1);
 	    }
-	    else { /* ((S2->l1==T2)&&(S2->l2==B_NODE)) */
-		return -1*ellSeg (S1->l2, S1->l1, T2);
-	    }
+	    /* ((S2->l1==T2)&&(S2->l2==B_NODE)) */
+	    return -1 * ellSeg(S1->l2, S1->l1, T2);
 	}
-	else if(S1->p.p2<S2->p.p2) {
+	if (S1->p.p2 < S2->p.p2) {
 	    if(S1->l2==T1)
 		return eqEndSeg (S2->l1, S1->l1, T1, T2);
-	    else
-		return -1*eqEndSeg (S2->l1, S1->l1, T1, T2);
-	}
-	else { /* S1->p.p2>S2->p.p2 */
-	    if(S2->l2==T2)
-		return eqEndSeg (S1->l1, S2->l1, T1, T2);
-	    else
-		return -1*eqEndSeg (S1->l1, S2->l1, T1, T2);
+	    return -1 * eqEndSeg(S2->l1, S1->l1, T1, T2);
 	}
+	/* S1->p.p2>S2->p.p2 */
+	if (S2->l2 == T2)
+	    return eqEndSeg(S1->l1, S2->l1, T1, T2);
+	return -1 * eqEndSeg(S1->l1, S2->l1, T1, T2);
     }
-    else if(S1->p.p2==S2->p.p1) {
+    if (S1->p.p2 == S2->p.p1) {
 	if (S1->l2 == S2->l1) return 0;
-	else if (S1->l2 == T2) return 1;
-	else return -1;
+	if (S1->l2 == T2) return 1;
+	return -1;
     }
-    else { /* S1->p.p1==S2->p.p2 */
-	if (S1->l1 == S2->l2) return 0;
-	else if (S1->l1 == T2) return 1;
-	else return -1;
-    }
-    assert(0);
-    return 0;
+    /* S1->p.p1==S2->p.p2 */
+    if (S1->l1 == S2->l2) return 0;
+    if (S1->l1 == T2) return 1;
+    return -1;
 }
 
 /* Function seg_cmp returns
@@ -733,7 +704,7 @@ static int
 seg_cmp(segment* S1, segment* S2)		
 {
     if(S1->isVert!=S2->isVert||S1->comm_coord!=S2->comm_coord) {
-	agerr (AGERR, "incomparable segments !! -- Aborting\n");
+	agerrorf("incomparable segments !! -- Aborting\n");
 	return -2;
     }
     if(S1->isVert)
@@ -745,14 +716,14 @@ seg_cmp(segment* S1, segment* S2)
 static int
 add_edges_in_G(channel* cp)
 {
-    int x,y;
-    segment** seg_list = cp->seg_list;
-    int size = cp->cnt;
+    seg_list_t *seg_list = &cp->seg_list;
+    const size_t size = seg_list_size(&cp->seg_list);
     rawgraph* G = cp->G;
 
-    for(x=0;x+1<size;x++) {
-	for(y=x+1;y<size;y++) {
-	    int cmp = seg_cmp(seg_list[x],seg_list[y]);
+    for (size_t x = 0; x + 1 < size; ++x) {
+	for (size_t y = x + 1; y < size; ++y) {
+	    const int cmp = seg_cmp(seg_list_get(seg_list, x),
+	                            seg_list_get(seg_list, y));
 	    if (cmp == -2) {
 		return -1;
 	    } else if (cmp > 0) {
@@ -778,7 +749,7 @@ add_np_edges (Dt_t* chans)
 	lp = ((chanItem*)l1)->chans;
 	for (l2 = dtflatten (lp); l2; l2 = dtlink(lp,l2)) {
 	    cp = (channel*)l2;
-	    if (cp->cnt)
+	    if (!seg_list_is_empty(&cp->seg_list))
 		if (add_edges_in_G(cp)) {
 		  return -1;
 		}
@@ -851,9 +822,9 @@ is_parallel(segment* s1, segment* s2)
 static int
 decide_point(pair *ret, segment* si, segment* sj, int dir1, int dir2)
 {
-    int prec, ans = 0, temp;
+    int prec = 0, ans = 0, temp;
     segment* np1;
-    segment* np2;
+    segment *np2 = NULL;
     
     while ((np1 = next_seg(si,dir1)) && (np2 = next_seg(sj,dir2)) &&
 	is_parallel(np1, np2)) {
@@ -994,7 +965,6 @@ removeEdge(segment* seg1, segment* seg2, int dir, maze* mp)
 static int
 addPEdges (channel* cp, maze* mp)
 {
-    int i,j;
     /* dir[1,2] are used to figure out whether we should use prev 
      * pointers or next pointers -- 0 : decrease, 1 : increase
      */
@@ -1007,61 +977,74 @@ addPEdges (channel* cp, maze* mp)
     int prec1, prec2;
     pair p;
     rawgraph* G = cp->G;
-    segment** segs = cp->seg_list;
+    seg_list_t *segs = &cp->seg_list;
 
-    for(i=0;i+1<cp->cnt;i++) {
-	for(j=i+1;j<cp->cnt;j++) {
+    for(size_t i = 0; i + 1 < seg_list_size(&cp->seg_list); ++i) {
+	for(size_t j = i + 1; j < seg_list_size(&cp->seg_list); ++j) {
 	    if (!edge_exists(G,i,j) && !edge_exists(G,j,i)) {
-		if (is_parallel(segs[i], segs[j])) {
+		if (is_parallel(seg_list_get(segs, i), seg_list_get(segs, j))) {
 		/* get_directions */
-		    if(segs[i]->prev==0) {
-			if(segs[j]->prev==0)
+		    if (seg_list_get(segs, i)->prev == 0) {
+			if (seg_list_get(segs, j)->prev == 0)
 			    dir = 0;
 			else
 			    dir = 1;
 		    }
-		    else if(segs[j]->prev==0) {
+		    else if (seg_list_get(segs, j)->prev == 0) {
 			dir = 1;
 		    }
 		    else {
-			if(segs[i]->prev->comm_coord==segs[j]->prev->comm_coord)
+			if (seg_list_get(segs, i)->prev->comm_coord ==
+			    seg_list_get(segs, j)->prev->comm_coord)
 			    dir = 0;
 			else
 			    dir = 1;
 		    }
 
-		    if (decide_point(&p, segs[i], segs[j], 0, dir) != 0) {
+		    if (decide_point(&p, seg_list_get(segs, i), seg_list_get(segs, j), 0, dir)
+		        != 0) {
 			return -1;
 		    }
 		    hops.a = p.a;
 		    prec1 = p.b;
-		    if (decide_point(&p, segs[i], segs[j], 1, 1-dir) != 0) {
+		    if (decide_point(&p, seg_list_get(segs, i), seg_list_get(segs, j), 1,
+		                     1 - dir) != 0) {
 			return -1;
 		    }
 		    hops.b = p.a;
 		    prec2 = p.b;
 
 		    if (prec1 == -1) {
-			set_parallel_edges (segs[j], segs[i], dir, 0, hops.a, mp);
-			set_parallel_edges (segs[j], segs[i], 1-dir, 1, hops.b, mp);
+			set_parallel_edges(seg_list_get(segs, j), seg_list_get(segs, i), dir, 0,
+			                   hops.a, mp);
+			set_parallel_edges(seg_list_get(segs, j), seg_list_get(segs, i), 1 - dir, 1,
+			                   hops.b, mp);
 			if(prec2==1)
-			    removeEdge (segs[i], segs[j], 1-dir, mp);
+			    removeEdge(seg_list_get(segs, i), seg_list_get(segs, j), 1 - dir, mp);
 		    } else if (prec1 == 0) {
 			if (prec2 == -1) {
-			    set_parallel_edges (segs[j], segs[i], dir, 0, hops.a, mp);
-			    set_parallel_edges (segs[j], segs[i], 1-dir, 1, hops.b, mp);
+			    set_parallel_edges(seg_list_get(segs, j), seg_list_get(segs, i), dir, 0,
+			                       hops.a, mp);
+			    set_parallel_edges(seg_list_get(segs, j), seg_list_get(segs, i), 1 - dir,
+			                       1, hops.b, mp);
 			} else if (prec2 == 0) {
-			    set_parallel_edges (segs[i], segs[j], 0, dir, hops.a, mp);
-			    set_parallel_edges (segs[i], segs[j], 1, 1-dir, hops.b, mp);
+			    set_parallel_edges(seg_list_get(segs, i), seg_list_get(segs, j), 0, dir,
+			                       hops.a, mp);
+			    set_parallel_edges(seg_list_get(segs, i), seg_list_get(segs, j), 1,
+			                       1 - dir, hops.b, mp);
 			} else if (prec2 == 1) {
-			    set_parallel_edges (segs[i], segs[j], 0, dir, hops.a, mp);
-			    set_parallel_edges (segs[i], segs[j], 1, 1-dir, hops.b, mp);
+			    set_parallel_edges(seg_list_get(segs, i), seg_list_get(segs, j), 0, dir,
+			                       hops.a, mp);
+			    set_parallel_edges(seg_list_get(segs, i), seg_list_get(segs, j), 1,
+			                       1 - dir, hops.b, mp);
 			}
 		    } else if (prec1 == 1) {
-			set_parallel_edges (segs[i], segs[j], 0, dir, hops.a, mp);
-			set_parallel_edges (segs[i], segs[j], 1, 1-dir, hops.b, mp);
+			set_parallel_edges(seg_list_get(segs, i), seg_list_get(segs, j), 0, dir,
+			                   hops.a, mp);
+			set_parallel_edges(seg_list_get(segs, i), seg_list_get(segs, j), 1, 1 - dir,
+			                   hops.b, mp);
 			if(prec2==-1)
-			    removeEdge (segs[i], segs[j], 1-dir, mp);
+			    removeEdge(seg_list_get(segs, i), seg_list_get(segs, j), 1 - dir, mp);
 		    }
 		}
 	    }
@@ -1124,17 +1107,15 @@ static double
 vtrack (segment* seg, maze* m)
 {
   channel* chp = chanSearch(m->vchans, seg);
-  double f = ((double)seg->track_no)/(chp->cnt+1); 
+  const double f = (double)seg->track_no / ((double)seg_list_size(&chp->seg_list) + 1);
   double left = chp->cp->bb.LL.x;
   double right = chp->cp->bb.UR.x;
   return left + f*(right-left);
 }
 
-static int
-htrack (segment* seg, maze* m)
-{
+static double htrack(segment *seg, maze *m) {
   channel* chp = chanSearch(m->hchans, seg);
-  double f = 1.0 - ((double)seg->track_no)/(chp->cnt+1); 
+  double f = 1.0 - (double)seg->track_no / ((double)seg_list_size(&chp->seg_list) + 1);
   double lo = chp->cp->bb.LL.y;
   double hi = chp->cp->bb.UR.y;
   return lo + f*(hi-lo);
@@ -1148,9 +1129,8 @@ addPoints(pointf p0, pointf p1)
     return p0;
 }
 
-static void
-attachOrthoEdges (Agraph_t* g, maze* mp, size_t n_edges, route* route_list, splineInfo *sinfo, epair_t es[], int doLbls)
-{
+static void attachOrthoEdges(maze *mp, size_t n_edges, route* route_list,
+                             splineInfo *sinfo, epair_t es[], bool doLbls) {
     int ipt;
     pointf* ispline = 0;
     size_t splsz = 0;
@@ -1169,7 +1149,7 @@ attachOrthoEdges (Agraph_t* g, maze* mp, size_t n_edges, route* route_list, spli
 	size_t npts = 1 + 3*rte.n;
 	if (npts > splsz) {
 		free (ispline);
-		ispline = N_GNEW(npts, pointf);
+		ispline = gv_calloc(npts, sizeof(pointf));
 		splsz = npts;
 	}
 	    
@@ -1208,7 +1188,7 @@ attachOrthoEdges (Agraph_t* g, maze* mp, size_t n_edges, route* route_list, spli
 	    fprintf(stderr, "ortho %s %s\n", agnameof(agtail(e)),agnameof(aghead(e)));
 	clip_and_install(e, aghead(e), ispline, npts, sinfo);
 	if (doLbls && (lbl = ED_label(e)) && !lbl->set)
-	    addEdgeLabels(g, e, p1, q1);
+	    addEdgeLabels(e);
     }
     free(ispline);
 }
@@ -1221,33 +1201,38 @@ edgeLen (Agedge_t* e)
     return (int)DIST2(p,q);
 }
 
-static int edgecmp(epair_t* e0, epair_t* e1)
-{
-    return e0->d - e1->d;
+static int edgecmp(const void *x, const void *y) {
+  const epair_t *e0 = x;
+  const epair_t *e1 = y;
+  if (e0->d > e1->d) {
+    return 1;
+  }
+  if (e0->d < e1->d) {
+    return -1;
+  }
+  return 0;
 }
 
-static boolean spline_merge(node_t * n)
+static bool spline_merge(node_t * n)
 {
     (void)n;
-    return FALSE;
+    return false;
 }
 
-static boolean swap_ends_p(edge_t * e)
+static bool swap_ends_p(edge_t * e)
 {
     (void)e;
-    return FALSE;
+    return false;
 }
 
-static splineInfo sinfo = { swap_ends_p, spline_merge, 1, 1 };
+static splineInfo sinfo = { swap_ends_p, spline_merge, true, true };
 
 /* orthoEdges:
  * For edges without position information, construct an orthogonal routing.
- * If doLbls is true, use edge label info when available to guide routing, 
+ * If useLbls is true, use edge label info when available to guide routing, 
  * and set label pos for those edges for which this info is not available.
  */
-void
-orthoEdges (Agraph_t* g, int doLbls)
-{
+void orthoEdges(Agraph_t *g, bool useLbls) {
     sgraph* sg;
     maze* mp;
     route* route_list;
@@ -1256,7 +1241,7 @@ orthoEdges (Agraph_t* g, int doLbls)
     Agedge_t* e;
     snode* sn;
     snode* dn;
-    epair_t* es = N_GNEW(agnedges(g), epair_t);
+    epair_t* es = gv_calloc(agnedges(g), sizeof(epair_t));
     cell* start;
     cell* dest;
     PointSet* ps = NULL;
@@ -1288,16 +1273,18 @@ orthoEdges (Agraph_t* g, int doLbls)
 		case 's' :
 		    odb_flags |= ODB_SGRAPH;    // emit search graph 
 		    break;
+		default:
+		    break;
 		}
 	    }
 	}
     }
 #endif
-    if (doLbls) {
-	agerr(AGWARN, "Orthogonal edges do not currently handle edge labels. Try using xlabels.\n");
-	doLbls = 0;
+    if (useLbls) {
+	agwarningf("Orthogonal edges do not currently handle edge labels. Try using xlabels.\n");
+	useLbls = false;
     }
-    mp = mkMaze (g, doLbls);
+    mp = mkMaze(g);
     sg = mp->sg;
 #ifdef DEBUG
     if (odb_flags & ODB_SGRAPH) emitSearchGraph (stderr, sg);
@@ -1326,9 +1313,9 @@ orthoEdges (Agraph_t* g, int doLbls)
 	}
     }
 
-    route_list = N_NEW (n_edges, route);
+    route_list = gv_calloc(n_edges, sizeof(route));
 
-    qsort(es, n_edges, sizeof(epair_t), (qsort_cmpf) edgecmp);
+    qsort(es, n_edges, sizeof(epair_t), edgecmp);
 
     gstart = sg->nnodes;
     PQgen (sg->nnodes+2);
@@ -1342,7 +1329,7 @@ orthoEdges (Agraph_t* g, int doLbls)
         start = CELL(agtail(e));
         dest = CELL(aghead(e));
 
-	if (doLbls && (lbl = ED_label(e)) && lbl->set) {
+	if (useLbls && (lbl = ED_label(e)) && lbl->set) {
 	}
 	else {
 	    if (start == dest)
@@ -1367,7 +1354,7 @@ orthoEdges (Agraph_t* g, int doLbls)
 #ifdef DEBUG
     if (odb_flags & ODB_ROUTE) emitGraph (stderr, mp, n_edges, route_list, es);
 #endif
-    attachOrthoEdges (g, mp, n_edges, route_list, &sinfo, es, doLbls);
+    attachOrthoEdges(mp, n_edges, route_list, &sinfo, es, useLbls);
 
 orthofinish:
     if (Concentrate)
@@ -1380,7 +1367,6 @@ orthofinish:
     free (es);
 }
 
-#ifdef DEBUG
 #include <common/arith.h>
 #define TRANS 10
 
@@ -1419,12 +1405,10 @@ static char* prolog2 =
 static char* epilog2 =
 "showpage\n\
 %%%%Trailer\n\
-%%%%BoundingBox: %d %d %d %d\n";
+%%%%BoundingBox: %.f %.f %.f %.f\n";
 
-static point
-coordOf (cell* cp, snode* np)
-{
-    point p;
+static pointf coordOf(cell *cp, snode *np) {
+    pointf p;
     if (cp->sides[M_TOP] == np) {
 	p.x = (cp->bb.LL.x + cp->bb.UR.x)/2;
 	p.y = cp->bb.UR.y;
@@ -1442,8 +1426,8 @@ coordOf (cell* cp, snode* np)
 	p.x = cp->bb.UR.x;
     }
     else {
-	agerr (AGERR, "Node not adjacent to cell -- Aborting\n");
-	exit(EXIT_FAILURE);
+	agerrorf("Node not adjacent to cell -- Aborting\n");
+	graphviz_exit(EXIT_FAILURE);
     }
     return p;
 }
@@ -1451,7 +1435,7 @@ coordOf (cell* cp, snode* np)
 static boxf
 emitEdge (FILE* fp, Agedge_t* e, route rte, maze* m, boxf bb)
 {
-    int x, y;
+    double x, y;
     boxf n = CELL(agtail(e))->bb;
     segment* seg = rte.segs;
     if (seg->isVert) {
@@ -1462,11 +1446,11 @@ emitEdge (FILE* fp, Agedge_t* e, route rte, maze* m, boxf bb)
 	y = htrack(seg, m);
 	x = (n.UR.x + n.LL.x)/2;
     }
-    bb.LL.x = MIN(bb.LL.x, SC*x);
-    bb.LL.y = MIN(bb.LL.y, SC*y);
-    bb.UR.x = MAX(bb.UR.x, SC*x);
-    bb.UR.y = MAX(bb.UR.y, SC*y);
-    fprintf (fp, "newpath %d %d moveto\n", SC*x, SC*y);
+    bb.LL.x = fmin(bb.LL.x, SC * x);
+    bb.LL.y = fmin(bb.LL.y, SC * y);
+    bb.UR.x = fmax(bb.UR.x, SC * x);
+    bb.UR.y = fmax(bb.UR.y, SC * y);
+    fprintf(fp, "newpath %.0f %.0f moveto\n", SC * x, SC * y);
 
     for (size_t i = 1;i<rte.n;i++) {
 	seg = rte.segs+i;
@@ -1476,11 +1460,11 @@ emitEdge (FILE* fp, Agedge_t* e, route rte, maze* m, boxf bb)
 	else {
 	    y = htrack(seg, m);
 	}
-	bb.LL.x = MIN(bb.LL.x, SC*x);
-	bb.LL.y = MIN(bb.LL.y, SC*y);
-	bb.UR.x = MAX(bb.UR.x, SC*x);
-	bb.UR.y = MAX(bb.UR.y, SC*y);
-	fprintf (fp, "%d %d lineto\n", SC*x, SC*y);
+	bb.LL.x = fmin(bb.LL.x, SC * x);
+	bb.LL.y = fmin(bb.LL.y, SC * y);
+	bb.UR.x = fmax(bb.UR.x, SC * x);
+	bb.UR.y = fmax(bb.UR.y, SC * y);
+	fprintf(fp, "%.0f %.0f lineto\n", SC * x, SC * y);
     }
 
     n = CELL(aghead(e))->bb;
@@ -1492,61 +1476,64 @@ emitEdge (FILE* fp, Agedge_t* e, route rte, maze* m, boxf bb)
 	y = htrack(seg, m);
 	x = (n.LL.x + n.UR.x)/2;
     }
-    bb.LL.x = MIN(bb.LL.x, SC*x);
-    bb.LL.y = MIN(bb.LL.y, SC*y);
-    bb.UR.x = MAX(bb.UR.x, SC*x);
-    bb.UR.y = MAX(bb.UR.y, SC*y);
-    fprintf (fp, "%d %d lineto stroke\n", SC*x, SC*y);
+    bb.LL.x = fmin(bb.LL.x, SC * x);
+    bb.LL.y = fmin(bb.LL.y, SC * y);
+    bb.UR.x = fmax(bb.UR.x, SC * x);
+    bb.UR.y = fmax(bb.UR.y, SC * y);
+    fprintf(fp, "%.0f %.0f lineto stroke\n", SC * x, SC * y);
 
     return bb;
 }
 
-static void
-emitSearchGraph (FILE* fp, sgraph* sg)
-{
+/**
+ * @brief dumps in dot format @ref snode::cells and @ref edges of
+ * @ref sgraph for debugging
+ *
+ * The routine uses coordinates of @ref cells calculated
+ * from @ref gcells.
+ * Coordinates of @ref gcellg are calculated by original
+ * specified graph layout engine.
+ */
+
+static DEBUG_FN void emitSearchGraph(FILE *fp, sgraph *sg) {
     cell* cp;
     snode* np;
     sedge* ep;
-    point p;
+    pointf p;
     int i;
     fputs ("graph G {\n", fp);
     fputs (" node[shape=point]\n", fp);
+    fputs (" layout=neato\n", fp);
     for (i = 0; i < sg->nnodes; i++) {
 	np = sg->nodes+i;
 	cp = np->cells[0];
 	if (cp == np->cells[1]) {
-	    pointf pf = midPt (cp);
-	    p.x = pf.x;
-	    p.y = pf.y;
+	    p = midPt(cp);
 	}
 	else {
 	    if (IsNode(cp)) cp = np->cells[1];
 	    p = coordOf (cp, np);
 	}
-	fprintf (fp, "  %d [pos=\"%d,%d\"]\n", i, p.x, p.y);
+	fprintf (fp, "  %d [pos=\"%.0f,%.0f!\"]\n", i, p.x, p.y);
     }
     for (i = 0; i < sg->nedges; i++) {
 	ep = sg->edges+i;
-	fprintf (fp, "  %d -- %d[len=\"%f\"]\n", ep->v1, ep->v2, ep->weight);
+	fprintf (fp, "  %d -- %d[label=\"%f\"]\n", ep->v1, ep->v2, ep->weight);
     }
     fputs ("}\n", fp);
 }
 
-static void
-emitGraph (FILE* fp, maze* mp, size_t n_edges, route* route_list, epair_t es[])
-{
-    boxf bb, absbb;
-    box bbox;
-
-    absbb.LL.x = absbb.LL.y = MAXDOUBLE;
-    absbb.UR.x = absbb.UR.y = -MAXDOUBLE;
+static DEBUG_FN void emitGraph(FILE *fp, maze *mp, size_t n_edges,
+                               route *route_list, epair_t es[]) {
+    boxf absbb = {.LL = {.x = DBL_MAX, .y = DBL_MAX},
+                  .UR = {.x = -DBL_MAX, .y = -DBL_MAX}};
 
     fprintf (fp, "%s", prolog2);
     fprintf (fp, "%d %d translate\n", TRANS, TRANS);
 
     fputs ("0 0 1 setrgbcolor\n", fp);
     for (int i = 0; i < mp->ngcells; i++) {
-      bb = mp->gcells[i].bb;
+      const boxf bb = mp->gcells[i].bb;
       fprintf (fp, "%f %f %f %f node\n", bb.LL.x, bb.LL.y, bb.UR.x, bb.UR.y);
     }
 
@@ -1556,18 +1543,18 @@ emitGraph (FILE* fp, maze* mp, size_t n_edges, route* route_list, epair_t es[])
     
     fputs ("0.8 0.8 0.8 setrgbcolor\n", fp);
     for (int i = 0; i < mp->ncells; i++) {
-      bb = mp->cells[i].bb;
+      const boxf bb = mp->cells[i].bb;
       fprintf (fp, "%f %f %f %f cell\n", bb.LL.x, bb.LL.y, bb.UR.x, bb.UR.y);
-      absbb.LL.x = MIN(absbb.LL.x, bb.LL.x);
-      absbb.LL.y = MIN(absbb.LL.y, bb.LL.y);
-      absbb.UR.x = MAX(absbb.UR.x, bb.UR.x);
-      absbb.UR.y = MAX(absbb.UR.y, bb.UR.y);
+      absbb.LL.x = fmin(absbb.LL.x, bb.LL.x);
+      absbb.LL.y = fmin(absbb.LL.y, bb.LL.y);
+      absbb.UR.x = fmax(absbb.UR.x, bb.UR.x);
+      absbb.UR.y = fmax(absbb.UR.y, bb.UR.y);
     }
 
-    bbox.LL.x = absbb.LL.x + TRANS;
-    bbox.LL.y = absbb.LL.y + TRANS;
-    bbox.UR.x = absbb.UR.x + TRANS;
-    bbox.UR.y = absbb.UR.y + TRANS;
+    const boxf bbox = {
+      .LL = {.x = absbb.LL.x + TRANS,
+             .y = absbb.LL.y + TRANS},
+      .UR = {.x = absbb.UR.x + TRANS,
+             .y = absbb.UR.y + TRANS}};
     fprintf (fp, epilog2, bbox.LL.x, bbox.LL.y,  bbox.UR.x, bbox.UR.y);
 }
-#endif
