@@ -3,14 +3,14 @@
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/epl-v10.html
+ * https://www.eclipse.org/legal/epl-v10.html
  *
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
 
 #define STANDALONE
 #include <cgraph/agxbuf.h>
-#include <cgraph/alloc.h>
+#include <cgraph/startswith.h>
 #include <sparse/general.h>
 #include <sparse/DotIO.h>
 #include <sparse/clustering.h>
@@ -20,6 +20,8 @@
 #include <sparse/colorutil.h>
 #include <stdbool.h>
 #include <string.h>
+#include <util/alloc.h>
+#include <util/unreachable.h>
 
 typedef struct {
     Agrec_t h;
@@ -151,7 +153,7 @@ SparseMatrix SparseMatrix_import_dot(Agraph_t *g, int dim,
 	  nitems = sscanf(pval, "%lf,%lf", &xx, &yy);
 	  if (nitems != 2) {
             has_positions = false;
-            agerr(AGERR, "Node \"%s\" pos has %d < 2 values", agnameof(n), nitems);
+            agerrorf("Node \"%s\" pos has %d < 2 values", agnameof(n), nitems);
 	  }
 	  (*x)[i*dim] = xx;
 	  (*x)[i*dim+1] = yy;
@@ -159,7 +161,7 @@ SparseMatrix SparseMatrix_import_dot(Agraph_t *g, int dim,
 	  nitems = sscanf(pval, "%lf,%lf,%lf", &xx, &yy, &zz);
 	  if (nitems != 3) {
             has_positions = false;
-            agerr(AGERR, "Node \"%s\" pos has %d < 3 values", agnameof(n), nitems);
+            agerrorf("Node \"%s\" pos has %d < 3 values", agnameof(n), nitems);
 	  }
 	  (*x)[i*dim] = xx;
 	  (*x)[i*dim+1] = yy;
@@ -168,7 +170,7 @@ SparseMatrix SparseMatrix_import_dot(Agraph_t *g, int dim,
 	  nitems = sscanf(pval, "%lf,%lf,%lf,%lf", &xx, &yy, &zz,&ww);
 	  if (nitems != 4) {
             has_positions = false;
-            agerr(AGERR, "Node \"%s\" pos has %d < 4 values", agnameof(n), nitems);
+            agerrorf("Node \"%s\" pos has %d < 4 values", agnameof(n), nitems);
 	  }
 	  (*x)[i*dim] = xx;
 	  (*x)[i*dim+1] = yy;
@@ -177,6 +179,7 @@ SparseMatrix SparseMatrix_import_dot(Agraph_t *g, int dim,
 	} else if (dim == 1){
 	  nitems = sscanf(pval, "%lf", &xx);
 	  if (nitems != 1){
+	    SparseMatrix_delete(A);
 	    A = NULL;
 	    goto done;
           }
@@ -186,7 +189,7 @@ SparseMatrix SparseMatrix_import_dot(Agraph_t *g, int dim,
 	}
       } else {
         has_positions = false;
-	agerr(AGERR, "Node \"%s\" lacks position info", agnameof(n));
+	agerrorf("Node \"%s\" lacks position info", agnameof(n));
       }
     }
     if (!has_positions) {
@@ -195,7 +198,7 @@ SparseMatrix SparseMatrix_import_dot(Agraph_t *g, int dim,
     }
   }
   else if (x)
-    agerr (AGERR, "Error: graph %s has missing \"pos\" information", agnameof(g));
+    agerrorf("Error: graph %s has missing \"pos\" information", agnameof(g));
 
   size_t sz = sizeof(double);
   if (format == FORMAT_CSR) A = SparseMatrix_from_coordinate_arrays(nedges, nnodes, nnodes, I, J, val, type, sz);
@@ -237,18 +240,14 @@ int Import_dot_splines(Agraph_t* g, int *ne, char ***xsplines){
   sym = agattr(g, AGEDGE, "pos", 0); 
   if (!sym) return 0;
  
-  if (!(*xsplines)) *xsplines = malloc(sizeof(char*)*nedges);
+  *xsplines = gv_calloc(nedges, sizeof(char*));
 
   i = 0;
   for (n = agfstnode (g); n; n = agnxtnode (g, n)) {
     for (e = agfstout (g, n); e; e = agnxtout (g, e)) {
       /* edge weight */
-      if (sym) {
-	char *pos = agxget (e, sym);
-	(*xsplines)[i] = strdup(pos);
-      } else {
-	(*xsplines)[i] = NULL;
-      }
+      char *pos = agxget (e, sym);
+      (*xsplines)[i] = strdup(pos);
       i++;
     }
   }
@@ -303,7 +302,7 @@ SparseMatrix Import_coord_clusters_from_dot(Agraph_t* g, int maxcluster, int dim
   int MAX_GRPS, MIN_GRPS;
   bool noclusterinfo = false;
   bool first = true;
-  float *pal;
+  const float *pal;
   int max_color = MAX_COLOR;
 
   switch (default_color_scheme){
@@ -323,11 +322,9 @@ SparseMatrix Import_coord_clusters_from_dot(Agraph_t* g, int maxcluster, int dim
     pal = &(palette_pastel[0][0]);
     break;
   case COLOR_SCHEME_SEQUENTIAL_SINGLEHUE_RED:
-    fprintf(stderr," HERE!\n");
     pal = &(palette_sequential_singlehue_red[0][0]);
     break;
   case COLOR_SCHEME_SEQUENTIAL_SINGLEHUE_RED_LIGHTER:
-    fprintf(stderr," HERE!\n");
     pal = &(palette_sequential_singlehue_red_lighter[0][0]);
     break;
   case COLOR_SCHEME_PRIMARY:
@@ -397,7 +394,7 @@ SparseMatrix Import_coord_clusters_from_dot(Agraph_t* g, int maxcluster, int dim
     int gid = 1;  
     memset (*clusters, 0, sizeof(int)*nnodes);
     for (sg = agfstsubg(g); sg; sg = agnxtsubg(sg)) {
-      if (strncmp(agnameof(sg), "cluster", 7)) continue;
+      if (!startswith(agnameof(sg), "cluster")) continue;
       gid++;
       for (n = agfstnode(sg); n; n = agnxtnode (sg, n)) {
         i = ND_id(n);
@@ -448,7 +445,7 @@ SparseMatrix Import_coord_clusters_from_dot(Agraph_t* g, int maxcluster, int dim
       modularity_clustering(A, false, maxcluster,
 		    &nc, clusters, &modularity);
     } else {
-      assert(0);
+      UNREACHABLE();
     }
     for (i = 0; i < nnodes; i++) (*clusters)[i]++;/* make into 1 based */
     for (n = agfstnode (g); n; n = agnxtnode (g, n)) {
@@ -626,7 +623,7 @@ void attached_clustering(Agraph_t* g, int maxcluster, int clustering_scheme){
       modularity_clustering(A, false, maxcluster,
 			    &nc, &clusters, &modularity);
     } else {
-      assert(0);
+      UNREACHABLE();
     }
     for (i = 0; i < nnodes; i++) (clusters)[i]++;/* make into 1 based */
     for (n = agfstnode (g); n; n = agnxtnode (g, n)) {
